-- RBT-TEST - ДПРД для модуля dblink
-- Тест написан для основного сервера, работающем на порту 5998 и для удаленного, работающем на 5999
--
\i 'support/init.sql'
--
-- Подключаемся на удаленный сервер
\connect postgres postgres - 5999
\i 'support/init.sql'
--
ALTER USER u_0_00 PASSWORD '123456';
ALTER USER postgres PASSWORD '123456';
-- Создаём тестовую таблицу
CREATE TABLE "Таблица"("Идентификатор" INTEGER, "Значение" TEXT);
-- Наполняем таблицу 'Таблица'
INSERT INTO "Таблица" VALUES(1, 'Значение1');
INSERT INTO "Таблица" VALUES(2, 'Значение2');
INSERT INTO "Таблица" VALUES(3, 'Значение3');
-- Выдаем права на таблицу
GRANT ALL ON TABLE "Таблица" TO PUBLIC;
-- Создаём хранимую процедуру, необходимую для тестирования асинхронных соединений
CREATE OR REPLACE FUNCTION async_function(IN INTEGER) RETURNS TEXT AS 
$$ SELECT pg_sleep($1) || 'успех'; $$
LANGUAGE SQL;
-- Выдаём права на исполнение пользователю PUBLIC
GRANT EXECUTE ON FUNCTION async_function(INTEGER) TO PUBLIC;
--
-- Подключаемся на текущий кластер 
\connect mtest u_0_00 - 5998
-- Ожидается ошибка, необходимы права суперпользователя
CREATE EXTENSION dblink;
-- Подключаемся как postgres и повторяем регистрацию dblink
\connect - postgres
-- Ожидается успех
CREATE EXTENSION dblink;
-- Подключаемся как пользователь 'u_0_00'
\connect - u_0_00
--
-- ========== 1. dblink_connect() -- открывает соединение к БД (доступно только администраторам) ==========
-- dblink_connect('имя_подключения', 'строка_подключения')
-- dblink_connect('строка_подключения')
-- ========== 2. dblink_disconnect() -- закрывает соединение к БД  ==========
-- dblink_disconnect('имя_подключения', 'строка_подключения')
-- dblink_disconnect('строка_подключения')
--
-- Ожидается ошибка, т.к. данной функции необходимо указывать пароль
SELECT dblink_connect('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00');
-- Подключаемся как postgres и повторяем попытку
\connect - postgres
-- Ожидается успех
SELECT dblink_connect('connection1', 'host = 127.0.0.1 port = 5999 dbname = mtest user = postgres password = 123456');
-- Открываем безымянное соединение
-- Ожидается успех
SELECT dblink_connect('host = 127.0.0.1 port = 5999 dbname = mtest password = 123456');
-- Закрываем соединение
-- Ожидается ошибка, т.к. соединения 'connection' не было открыто
SELECT dblink_disconnect('connection');
-- Ожидается успех
SELECT dblink_disconnect('connection1');
-- Закрываем безымянное соединение
-- Ожидается успех
SELECT dblink_disconnect();
--
-- ========== 3. dblink_connect_u() -- открывает соединение к БД (доступно пользователям) ==========
--
-- dblink_connect_u('имя_подключения', 'строка_подключения')
-- dblink_connect_u('строка_подключения')
-- Выдаём права на исполнение функции dblink_connect_u() пользователю 'u_0_00'
GRANT EXECUTE ON FUNCTION dblink_connect_u(TEXT) TO u_0_00;
GRANT EXECUTE ON FUNCTION dblink_connect_u(TEXT, TEXT) TO u_0_00;
-- Подключаемся как пользователь 'u_0_00' и выполняем попытку подключения к БД 'mtest'
\connect - u_0_00
-- Ожидается успех
SELECT dblink_connect_u('connection2', 'host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Открытие безымянного соединения
-- Ожидается успех
SELECT dblink_connect_u('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Закрытие соединения 'connection2'
-- Ожидается успех
SELECT dblink_disconnect('connection2');
-- Закрытие безымянного соединения
-- Ожидается успех
SELECT dblink_disconnect();
--
-- ========== 4. dblink() -- открывает соединение к БД и исполняет запрос ==========
--
-- dblink('имя_подключения', 'запрос' [, 'флаг']) 
-- dblink('строка_подключения', 'запрос' [, 'флаг'])
-- dblink('запрос' [, флаг]) 
--
-- Создаём подключение к БД 'mtest'
-- Ожидается успех
SELECT dblink_connect_u('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
SELECT dblink_connect_u('connection1', 'host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Выполняем запросы на удаленной БД 'mtest'
-- Ожидается 3 записи
SELECT * FROM dblink('connection1', 'SELECT * FROM "Таблица";', FALSE) AS "Результаты" ("Идентификатор" INTEGER, "Значение" TEXT);
-- Ожидается 2 записи
SELECT * FROM dblink('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456', 'SELECT * FROM "Таблица" WHERE "Идентификатор" <= 2;', FALSE)
AS "Результаты2" ("Идентификатор" INTEGER, "Значение" TEXT);
-- Выполняем запрос по анонимному подключению на удалённую БД 'mtest'
-- Ожидается 1 запись
SELECT * FROM dblink('SELECT * FROM "Таблица" WHERE "Идентификатор" % 2 = 0;', FALSE) AS "Результаты3" ("Идентификатор" INTEGER, "Значение" TEXT);
-- Закрываем подключения
SELECT dblink_disconnect('connection1');
SELECT dblink_disconnect();
--
-- ========== 5. dblink_exec() -- открывает соединение к БД и исполняет команду ==========
--
-- dblink_exec('имя_подключения', 'команда' [, флаг])
-- dblink_exec('строка_подключения', 'команда' [, флаг])
-- dblink_exec('команда' [, флаг])
--
-- Создаём подключения
SELECT dblink_connect_u('connection1', 'host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
SELECT dblink_connect_u('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Исполняем команды на удалённой БД 'mtest'
-- Ожидается успех
SELECT dblink_exec('connection1', 'INSERT INTO "Таблица" VALUES (4, ''Значение4'');', FALSE);
-- Ожидается ошибка, связанная с несовместимости типов вставляемых данных с табличными
SELECT dblink_exec('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456', 'INSERT INTO "Таблица" VALUES(''Пять'', ''Значение5'');', FALSE);
-- Ожидается успех
SELECT dblink_exec('INSERT INTO "Таблица" VALUES(5, ''Значение5'');', FALSE);
-- Закрываем подключения
SELECT dblink_disconnect('connection1');
SELECT dblink_disconnect();
-- Убедимся в том, что в таблицу на удаленном кластере были добавлены записи
\connect mtest postgres - 5999
-- Ожидается 5 записей
SELECT * FROM "Таблица";
--
-- ========== 6. dblink_open() -- открывает курсор в удалённой БД ==========
-- dblink_open('имя_курсора', 'запрос' [, флаг])
-- dblink_open('имя_подключения', 'имя_курсора', 'запрос' [, флаг])
-- ========== 7. dblink_fetch() -- возвращает строки, хранимые в открытом курсоре ==========
-- dblink_fetch('имя_курсора', 'количество_строк' [, флаг])
-- dblink_fetch('имя_подключения', 'имя_курсора', 'количество_строк' [, флаг])
-- ========== 8. dblink_close() -- закрывает курсор в удалённой БД ==========
-- dblink_close('имя_курсора' [, флаг])
-- dblink_close('имя_подключения', 'имя_курсора' [, флаг])
--
-- Подключаемся в 'mtest' как пользователь 'u_0_00'
\connect mtest u_0_00 - 5998
-- Создаём подключения
SELECT dblink_connect_u('host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
SELECT dblink_connect_u('connection1', 'host = 127.0.0.1 port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Открываем курсоры
-- Ожидается успех
SELECT dblink_open('cursor_unnamed', 'SELECT * FROM "Таблица"', FALSE);
SELECT dblink_open('connection1', 'cursor', 'SELECT "Идентификатор" FROM "Таблица"', FALSE);
-- Выводим строки, хранящиеся в курсорах после выполнения dblink_open()
-- Ожидается 3 записи
SELECT * FROM dblink_fetch('cursor_unnamed', 3, FALSE) AS ("Идентификатор" INTEGER, "Значение" TEXT);
-- Ожидается 5 значений идентификаторов
SELECT * FROM dblink_fetch('connection1', 'cursor', 5, FALSE) AS ("Идентификатор" INTEGER);
-- Закрываем курсоры
-- Ожидается успех
SELECT dblink_close('cursor_unnamed', FALSE);
SELECT dblink_close('connection1', 'cursor', FALSE);
-- Закрываем несуществующий курсор
-- Ожидается ошибка
SELECT dblink_close('cursor1', FALSE);
-- Закрываем подключения
SELECT dblink_disconnect('connection1');
SELECT dblink_disconnect();
--
-- ========== 9. dblink_get_connections() -- возвращает список подключений к удалённым БД ==========
--
-- Убедимся в отсутствии активных подключений
-- Ожидается пустая строка
SELECT dblink_get_connections();
-- Создадим новые подключения к базе данных 'mtest'
SELECT dblink_connect_u('connection1', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
SELECT dblink_connect_u('connection2', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Убедимся, что было открыто 2 подключения
SELECT dblink_get_connections();
-- Закрываем подключение 'connection2'
SELECT dblink_disconnect('connection2');
-- Убедимся, что открытым осталось только подключение 'connection1'
SELECT dblink_get_connections();
-- Закрываем подключение 'connection1'
SELECT dblink_disconnect('connection1');
--
-- ========== 10. dblink_error_message() -- возвращает последнее ошибочное сообщение от подключения к БД ==========
-- dblink_error_message('имя_подключения') 
--
-- Создадим новое подключение
SELECT dblink_connect_u('connection1', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Попытаемся выполнить заранее ошибочный запрос
SELECT dblink('connection1', 'SELECT * FROM "Проверка2"', FALSE);
-- Просмотрим последнее ошибочное сообщение от подключения 'connection1'
SELECT dblink_error_message('connection1');
-- Закрываем подключение
SELECT dblink_disconnect('connection1');
--
-- ========== 11. dblink_send_query() -- посылает запрос на асинхронное выполнение ==========
-- dblink_send_query('имя_подключения', 'запрос') 
-- ========== 12. dblink_is_busy() -- возвращает TRUE, если асинхронный запрос выполняется и FALSE иначе ==========
-- dblink_is_busy('имя_подключения')
-- ========== 13. dblink_get_result() -- получить результат асинхронного запроса ==========
-- dblink_get_result('имя_подключения' [, флаг])
--
-- Открываем подключение
SELECT dblink_connect_u('connection1', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Посылаем асинхронный запрос на БД 'mtest -- вызываем функцию'
-- Ожидается 1 -- код успешного завершения
SELECT dblink_send_query('connection1', 'SELECT async_function(1)');
-- Проверим, выполняется ли асинхронный запрос
-- Ожидается 1 -- соединение занято
SELECT dblink_is_busy('connection1');
-- Засыпаем на 2 секунды
SELECT pg_sleep(2);
-- Проверим, завершилось ли асинхронный запрос
-- Ожидается 0 -- соединение свободно
SELECT dblink_is_busy('connection1');
-- Получаем результат асинхронного запроса
SELECT * FROM dblink_get_result('connection1', FALSE) AS "Результаты"(async_function TEXT);
-- Закрываем подключение
SELECT dblink_disconnect('connection1');
--
-- ========== 14. dblink_get_notify() -- возвращает асинхронные уведомления на подключении ==========
-- dblink_get_notify() 
-- dblink_get_notify('имя_подключения')
--
-- Открываем подключения
SELECT dblink_connect_u('connection1', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
SELECT dblink_connect_u('port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Устанавливаем слушателя на удалённой БД
SELECT dblink_exec('connection1', 'LISTEN listener');
SELECT dblink_exec('LISTEN listener2');
-- Убеждаемся, что не начались уведомления
SELECT notify_name FROM dblink_get_notify('connection1');
SELECT notify_name FROM dblink_get_notify();
-- Запускаем процесс уведомлений
SELECT dblink_exec('connection1', 'NOTIFY listener');
SELECT dblink_exec('NOTIFY listener2');
-- Убеждаемся, что сервер запустил уведомления
SELECT notify_name FROM dblink_get_notify('connection1');
SELECT notify_name FROM dblink_get_notify();
-- Закрываем подключения
SELECT dblink_disconnect('connection1');
SELECT dblink_disconnect();
--
-- ========== 15. dblink_cancel_query() -- отменить выполнение любого запроса ==========
-- dblink_cancel_query('имя_подключения')
--
-- Открываем подключение
SELECT dblink_connect_u('connection1', 'port = 5999 dbname = mtest user = u_0_00 password = 123456');
-- Посылаем асинхронный запрос на БД 'mtest -- вызываем функцию'
-- Ожидается 1 -- код успешного завершения
SELECT dblink_send_query('connection1', 'SELECT async_function(2)');
-- Проверим, выполняется ли асинхронный запрос
-- Ожидается 1 -- соединение занято
SELECT dblink_is_busy('connection1');
-- Отменяем выполнение запроса
SELECT dblink_cancel_query('connection1');
-- Закрываем подключение
SELECT dblink_disconnect('connection1');
--
-- ========== 16. dblink_get_pkey() -- получить позиции и имена первичных ключей отношения в локальной БД ==========
-- dblink_get_pkey('имя_отношения')
--
-- Cоздаём таблицу в БД 'mtest'
CREATE TABLE "Проверка2" ("Ключ1" INTEGER, "Ключ2" INTEGER, PRIMARY KEY("Ключ1", "Ключ2"));
-- Получаем значения первичных ключей из таблицы 'Проверка2'
SELECT * FROM dblink_get_pkey('Проверка2');
-- Удаляем таблицу 'Таблица'
DROP TABLE "Проверка2";
--
-- Выполняем очистку
-- Подключаемся на удаленный кластер
\connect postgres postgres - 5999
\i 'support/done.sql'
-- Подключаемся на основной кластер
\connect postgres postgres - 5998
\i 'support/done.sql'