-- RBT-TEST - ДПРД при работе с большими объектами
-- Проверка ДПРД при работе с большими объектами
--
\i 'support/init.sql'
--
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Проверяем, что большие объекты имеют атрибут мандатной метки
-- Ожидается что записи таблиц pg_largeobject и pg_largeobject_metadata защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'pg_largeobject'::regclass::oid or oid = 'pg_largeobject_metadata'::regclass::oid;
-- 
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB" (loid oid, fd integer);
--
-- Отключаем ДПРД и МПРД к созданной таблице
GRANT ALL ON "Проверка BLOB" TO PUBLIC;
\connect - postgres
ALTER TABLE "Проверка BLOB" SET MAC TO NULL;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Создаем объект
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200000);
SELECT * FROM "Проверка BLOB";
--
-- Начинаем транзакцию
BEGIN;
--
-- Открываем объект на чтение и запись
--   INV_READ  = 0x40000
--   INV_WRITE = 0x20000
UPDATE "Проверка BLOB" SET fd = lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Проверка записи, ожидается успех
SELECT lowrite(fd, '
Whose woods these are I think I know,
His house is in the village though.
He will not see me stopping here,
To watch his woods fill up with snow.

My little horse must think it queer,
To stop without a farmhouse near,
Between the woods and frozen lake,
The darkest evening of the year.

He gives his harness bells a shake,
To ask if there is some mistake.
The only other sound''s the sweep,
Of easy wind and downy flake.

The woods are lovely, dark and deep,
But I have promises to keep,
And miles to go before I sleep,
And miles to go before I sleep.

         -- Robert Frost
') FROM "Проверка BLOB";
--
-- Закрываем объект
-- Ожидается 0 - отсутствие ошибок, отрицательное значение - ошибка
SELECT lo_close(fd) FROM "Проверка BLOB";
--
-- Завершаем транзакцию
END;
--
-- Проверяем прямой доступ к данным объекта
-- Ожидается ошибка доступа
SELECT maclabel, pageno, data FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {0,0}
SELECT maclabel FROM pg_largeobject_metadata;
--
-- Проверяем чтение/запись для владельца
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Записываем символ
SELECT lowrite(fd, 'n') FROM "Проверка BLOB";
--
-- Читаем значение позиции
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -156, 2) FROM "Проверка BLOB";
--
-- Читаем 35 символов
-- Ожидается изменение буквы в слове на записанное значение
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Проверяем изменение размера объекта
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Увеличиваем
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Уменьшаем
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Проверяем возможность работы с объектом других пользователей
-- Подключаемся от имени пользователя u2_0_00
\connect - u2_0_00
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Открываем объект на чтение
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Читаем 35 символов
-- Ожидается ошибка доступа
SELECT loread(fd, 35) FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Записываем символ
-- Ожидается ошибка доступа
SELECT lowrite(fd, 'n') FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Увеличиваем
-- Ожидается ошибка доступа
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Уменьшаем
-- Ожидается ошибка доступа
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Предоставляем право SELECT
GRANT SELECT ON LARGE OBJECT 200000 to PUBLIC;
--
-- Подключаемся от имени пользователя u2_0_00
\connect - u2_0_00
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Открываем объект на чтение
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Читаем значение позиции
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Записываем символ
-- Ожидается ошибка доступа
SELECT lowrite(fd, 'n') FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Увеличиваем
-- Ожидается ошибка доступа
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
END;
--
-- Открываем объект на запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
--
-- Уменьшаем
-- Ожидается ошибка доступа
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Предоставляем право UPDATE
GRANT UPDATE ON LARGE OBJECT 200000 to PUBLIC;
--
-- Проверяем возможность работы с объектом других пользователей
-- Подключаемся от имени пользователя u2_0_00
\connect - u2_0_00
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Записываем символ
SELECT lowrite(fd, 'n') FROM "Проверка BLOB";
--
-- Читаем значение позиции
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -156, 2) FROM "Проверка BLOB";
--
-- Читаем 35 символов
-- Ожидается изменение буквы в слове на записанное значение
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Проверяем изменение размера объекта
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Увеличиваем
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Уменьшаем
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Проверяем возможность выгрузки объекта на диск
-- Ожидается ошибка - функция не существует
SELECT lo_export(loid, '/tmp/lotest.txt') FROM "Проверка BLOB";
--
--
\i 'support/done.sql'