-- RBT-TEST - ДПРД при использовании ролей (групп)
-- Проверка ролевого разграничения доступа
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
--
-- Отбираем права у PUBLIC на тестовую таблицу
REVOKE ALL ON "Проверка" FROM PUBLIC;
--
-- Добавим несколько записей от лица владельца таблицы "Проверка"
INSERT INTO "Проверка" VALUES (1, 'Первая запись ' || USER); 
INSERT INTO "Проверка" VALUES (2, 'Вторая запись ' || USER);
--
-- Просматриваем созданные записи
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Изначально все действия пользователю над таблицей "Проверка" запрещены
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
--
-- Создаем роль (группу) "Читающих" с наследованием прав
CREATE ROLE "Читающие" NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE;
--
-- И даем ей право выполнять операцию SELECT
GRANT SELECT ON "Проверка" TO "Читающие";
--
-- Даем пользователю u_1_10 возможность использовать эту роль
GRANT "Читающие" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
--
-- Создаем роль (группу) "Добавляющих" без наследования прав
CREATE ROLE "Добавляющие" NOSUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE;
--
-- И даем ей право выполнять операцию INSERT
GRANT INSERT ON "Проверка" TO "Добавляющие";
--
-- Даем пользователю u_1_10 возможность использовать эту роль
GRANT "Добавляющие" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается успех
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Проверяем результат выполнения добавления
-- Ожидается появление новой записи
SELECT * FROM "Проверка";
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
--
-- Создаем роль (группу) "Обновляющих"
CREATE ROLE "Обновляющие" NOSUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE;
--
-- И даем ей право выполнять операцию SELECT и UPDATE
GRANT SELECT ON "Проверка" TO "Обновляющие";
GRANT UPDATE ON "Проверка" TO "Обновляющие";
--
-- Даем роли "Добавляющие" возможность использовать эту роль
GRANT "Обновляющие" TO "Добавляющие";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 3;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 3;
--
-- Попытка выполнить INSERT
-- Ожидается успех
INSERT INTO "Проверка" VALUES (4, 'Запись пользователя u_1_10');
--
-- Устанавливаем используемую роль
SET ROLE "Обновляющие";
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается успех
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 3;
--
-- Проверяем результат выполнения обновления
SELECT * FROM "Проверка";
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 4;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (5, 'Запись пользователя u_1_10');
--
-- Подключаемся как суперпользователь postgres
\connect - postgres
--
-- Создаем роль (группу) "Удаляющих"
CREATE ROLE "Удаляющие" NOSUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE;
--
-- И даем ей право выполнять операцию DELETE
GRANT DELETE ON "Проверка" TO "Удаляющие";
--
-- Даем роли "Читающие" возможность использовать эту роль
GRANT "Удаляющие" TO "Читающие";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 10 WHERE "идентификатор" = 4;
--
-- Попытка выполнить DELETE
-- Ожидается успех
DELETE FROM "Проверка" WHERE "идентификатор" = 13;
--
-- Пороверяем результат выполнение операции
-- Ожидается удаление записи с идентификатором 13
SELECT * FROM "Проверка";
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (6, 'Запись пользователя u_1_10');
--
-- Удаляем созданные объекты
\connect - postgres
REVOKE "Читающие" FROM u_1_10;
REVOKE ALL ON "Проверка" FROM "Читающие";
DROP ROLE "Читающие";
REVOKE "Добавляющие" FROM u_1_10;
REVOKE ALL ON "Проверка" FROM "Добавляющие";
DROP ROLE "Добавляющие";
REVOKE ALL ON "Проверка" FROM "Обновляющие";
DROP ROLE "Обновляющие";
REVOKE ALL ON "Проверка" FROM "Удаляющие";
DROP ROLE "Удаляющие";
--
\i 'support/done.sql'
