-- RBT-TEST - ДПРД для схем
-- Проверка дискреционного разграничения доступа для схем
-- Права доступа для схем: CREATE, USAGE
--
\i 'support/init.sql'
--
-- Администратор кластера postgres дает права пользователю u_0_00 на создание объектов в базе данных
\connect - postgres
GRANT CREATE ON DATABASE mtest TO u_0_00;
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Создаем схему
CREATE SCHEMA "Схема";
--
-- Владельцу все действия разрешены
--
-- Проверка выполнения CREATE
-- ожидается успех
CREATE TABLE "Схема"."Таблица" (id integer NOT NULL);
--
-- Проверяем наличие созданной таблицы
SELECT relname AS "Наименование таблицы" from pg_class WHERE oid = 'Схема.Таблица'::regclass::oid;
--
-- Проверка обращения к объектам схемы (USAGE)
-- ожидается успех
SELECT * FROM "Схема"."Таблица";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения CREATE
-- ожидается ошибка доступа
CREATE TABLE "Схема"."Таблица1" (id integer NOT NULL);
--
-- Проверка обращения к объектам схемы (USAGE)
-- ожидается ошибка доступа
SELECT * FROM "Схема"."Таблица";
--
-- Владелец u_0_00 дает право CREATE для пользователя u_1_10
\connect - u_0_00
GRANT CREATE ON SCHEMA "Схема" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения CREATE
-- ожидается успех
CREATE TABLE "Схема"."Таблица1" (id integer NOT NULL);
--
-- Проверяем наличие созданной таблицы из под администратора postgres
\connect - postgres
SELECT relname AS "Наименование таблицы" from pg_class WHERE oid = 'Схема.Таблица1'::regclass::oid;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка обращения к объектам схемы (USAGE)
-- ожидается ошибка доступа
SELECT * FROM "Схема"."Таблица";
--
-- Владелец u_0_00 дает право USAGE для пользователя u_1_10 и отбирает CREATE
-- права на таблицу у пользователя u_1_10 нет
\connect - u_0_00
GRANT USAGE ON SCHEMA "Схема" TO u_1_10;
REVOKE CREATE ON SCHEMA "Схема" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения CREATE
-- ожидается ошибка доступа
CREATE TABLE "Схема"."Таблица2" (id integer NOT NULL);
--
-- Проверка обращения к объектам схемы (USAGE)
-- ожидается ошибка доступа на чтение из таблицы
SELECT * FROM "Схема"."Таблица";
--
-- Владелец u_0_00 дает право SELECT на таблицу для пользователя u_1_10
\connect - u_0_00
GRANT SELECT ON TABLE "Схема"."Таблица" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка обращения к объектам схемы (USAGE)
-- ожидается успех
SELECT * FROM "Схема"."Таблица";
--
\i 'support/done.sql'
