-- RBT-TEST - ДПРД для последовательностей
-- Проверка дискреционного разграничения доступа для последовательностей
-- Права доступа для последовательностей: USAGE, SELECT, UPDATE
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Общая подготовка
\connect - postgres
--
-- Создаем последовательность
CREATE SEQUENCE "Последовательность"
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Отключаем мандатный контроль доступа
-- Ожидается успех в случае установленного конфигурационного параметра ac_enable_sequence_mac
-- иначе ожидается ошибка доступа
ALTER TABLE "Последовательность" SET MAC TO NULL;
--
-- Владельцу все действия разрешены
--
-- Проверка выполнения SELECT
-- ожидается успех
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается успех
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается успех
SELECT currval('"Последовательность"');
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Изначально все действия пользователю над вновь созданной последовательностью запрещены
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается ошибка доступа
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка доступа
SELECT currval('"Последовательность"');
--
-- Администратор postgres дает право SELECT для пользователя u_1_10
-- все действия по получению данных разрешены
\connect - postgres
GRANT SELECT ON SEQUENCE "Последовательность" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения SELECT
-- ожидается успех
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается ошибка доступа
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка (не имеет смысла без возможности вызова nextval)
SELECT currval('"Последовательность"');
--
-- Администратор postgres отбирает право SELECT и дает право USAGE для пользователя u_1_10
\connect - postgres
REVOKE SELECT ON SEQUENCE "Последовательность" FROM u_1_10;
GRANT USAGE ON SEQUENCE "Последовательность" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается ошибка доступа
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается успех
SELECT currval('"Последовательность"');
--
-- Администратор postgres отбирает право USAGE и дает право UPDATE для пользователя u_1_10
\connect - postgres
REVOKE USAGE ON SEQUENCE "Последовательность" FROM u_1_10;
GRANT UPDATE ON SEQUENCE "Последовательность" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Проверка выполнения SELECT
-- ожидается ошибка доступа
SELECT * FROM "Последовательность";
--
-- Проверка вызова функции nextval
-- ожидается успех
SELECT nextval('"Последовательность"');
--
-- Проверка вызова функции setval
-- ожидается успех
SELECT setval('"Последовательность"', 100);
--
-- Проверка вызова функции currval
-- ожидается ошибка доступа
SELECT currval('"Последовательность"');
--
\i 'support/done.sql'
