-- RBT-TEST - ДПРД для таблиц
-- Проверка дискреционного разграничения доступа для таблиц
-- Права доступа для таблиц: SELECT, INSERT, UPDATE, DELETE, REFERENCES, TRIGGER
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'

--
-- Общая подготовка
\connect - postgres
--
-- Отбираем права у PUBLIC на тестовую таблицу
REVOKE ALL ON "Проверка" FROM PUBLIC;
--
-- Создаем триггерную функцию
CREATE FUNCTION trigger_func() RETURNS trigger LANGUAGE plpgsql AS '
BEGIN
	RAISE NOTICE ''trigger_func() called: action = %, when = %, level = %'', TG_OP, TG_WHEN, TG_LEVEL;
	RETURN NULL;
END;';
--
-- Добавим несколько записей от лица владельца таблицы "Проверка"
INSERT INTO "Проверка" VALUES (1, 'Первая запись ' || USER); 
INSERT INTO "Проверка" VALUES (2, 'Вторая запись ' || USER);
--
-- Просматриваем созданные записи
SELECT * FROM "Проверка";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Создаем проверочную таблицу для проверки создания ограничений
CREATE TABLE "Проверка2" ("идентификатор2" INTEGER PRIMARY KEY, "ключ" INTEGER);
--
-- Изначально все действия пользователю над таблицей "Проверка" запрещены
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres дает право SELECT для пользователя u_1_10
\connect - postgres
GRANT SELECT ON "Проверка" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции INSERT
-- и отбирает право на операцию SELECT
\connect - postgres
GRANT INSERT ON "Проверка" TO u_1_10;
REVOKE SELECT ON "Проверка" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается успех
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Переподключиться от имени администратора postgres
\connect - postgres
--
-- Убедимся, что в таблице "Проверка" появилась запись, занесенная пользователем u_1_10
SELECT * FROM "Проверка";
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции UPDATE
-- и отбирает право на операцию INSERT
GRANT UPDATE ON "Проверка" TO u_1_10;
REVOKE INSERT ON "Проверка" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа из-за отсутствия права SELECT
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 3;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (4, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции SELECT
\connect - postgres
GRANT SELECT ON "Проверка" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить UPDATE
-- Ожидается успех, так как разрешены и SELECT и UPDATE
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- В таблице "Проверка" идентификатор у записи пользователя увеличился на единицу
SELECT * FROM "Проверка";
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции DELETE
-- и отбирает право на операцию UPDATE и SELECT
\connect - postgres
GRANT DELETE ON "Проверка" TO u_1_10;
REVOKE UPDATE ON "Проверка" FROM u_1_10;
REVOKE SELECT ON "Проверка" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 4;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа из-за отсутствия права SELECT
DELETE FROM "Проверка" WHERE "идентификатор" = 4;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (5, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции SELECT
\connect - postgres
GRANT SELECT ON "Проверка" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить DELETE
-- Ожидается успех, так как разрешены и SELECT и DELETE
DELETE FROM "Проверка" WHERE "идентификатор" = 4;
--
-- В таблице "Проверка" запись пользователя была удалена
SELECT * FROM "Проверка";
--
-- Администратор postgres предоставляет пользователю u_1_10 право на создание триггеров
-- и отбирает право на операцию DELETE и SELECT
\connect - postgres
GRANT TRIGGER ON "Проверка" TO u_1_10;
REVOKE DELETE ON "Проверка" FROM u_1_10;
REVOKE SELECT ON "Проверка" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается успех
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается ошибка доступа
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
-- Администратор postgres предоставляет пользователю u_1_10 право на создание ссылки на поля таблицы
-- и отбирает право на операцию создания триггеров
\connect - postgres
GRANT REFERENCES ON "Проверка" TO u_1_10;
REVOKE TRIGGER ON "Проверка" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Проверка";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Проверка" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Проверка" VALUES (3, 'Запись пользователя u_1_10');
--
-- Попытка создать триггер
-- Ожидается ошибка доступа
CREATE TRIGGER before_ins_stmt_trig BEFORE INSERT ON "Проверка"
FOR EACH STATEMENT EXECUTE PROCEDURE trigger_func();
--
-- Попытка создать ограничение ссылающееся на таблицу
-- Ожидается успех
ALTER TABLE "Проверка2" ADD CONSTRAINT "Ссылка" FOREIGN KEY ("ключ") REFERENCES "Проверка" ("идентификатор")
   ON UPDATE NO ACTION ON DELETE NO ACTION;
--
\i 'support/done.sql'
