-- RBT-TEST - ДПРД для типов и доменов
-- Проверка дискреционного разграничения доступа для типов и доменов
-- Права доступа для типов и доменов - USAGE
--
\i 'support/init.sql'
--
-- ========== Проверка композитных типов (Composite Type) ==========
--
--
\connect - postgres
-- Создаём пользовательский тип 'Адрес'
CREATE TYPE "Адрес" AS ("Страна" TEXT, "Город" TEXT, "Улица" TEXT, "Дом" INTEGER, "Квартира" INTEGER);
-- Проверяем факт создания типа
-- Ожидается одна строка
SELECT typname FROM pg_type WHERE typname = 'Адрес'; 
-- Изымаем все права у PUBLIC
REVOKE ALL ON TYPE "Адрес" FROM PUBLIC;
--
\connect - u_0_00
-- Проверка приведения к типу 'Адрес'
-- Ожидается успех
SELECT ('Россия', 'Москва', 'Тверская', 7, 10)::"Адрес";
-- Ожидается ошибка
SELECT ('Россия', 'Москва', 'Тверская', 7, 'Десять')::"Адрес";
-- Проверяется возможность использования пользователем типа в таблице
-- Ожидается ошибка
CREATE TABLE "Место жительства"("ФИО" TEXT, "Адрес места жительства" "Адрес");
--
\connect - postgres
-- Выдаём права пользователю u_0_00
GRANT ALL ON TYPE "Адрес" TO u_0_00;
--
\connect - u_0_00
-- Проверяем возможность использовать тип при создании таблиц
-- Ожидается успех
CREATE TABLE "Место жительства"("ФИО" TEXT, "Адрес места жительства" "Адрес");
-- Вставка данных в таблицу
-- Ожидается успех
INSERT INTO "Место жительства" VALUES ('Иванов И.И.', ROW('Россия', 'Москва', 'Тверская', 7, 10));
-- Убеждаемся в успешном добавлении записи
-- Ожидается 1 запись
SELECT * FROM "Место жительства";
--
\connect - postgres
-- Удаляем созданную таблицу и тип
DROP TABLE "Место жительства";
DROP TYPE "Адрес";
--
--
-- ========== Проверка типа перечисления (Enumeration Type) ==========
--
--
-- Создаём тип перечисления
CREATE TYPE "Качество" AS ENUM('неудовлетворительное', 'удовлетворительное', 'хорошее', 'отличное');
-- Удостоверяемся, что тип был создан
-- Ожидается одна строка с именем типа
SELECT typname FROM pg_type WHERE typnmae = 'Качество';
-- Создаём таблицу 'Товар', в одном столбце которой используется созданный тип
CREATE TABLE "Товар"("Наименование" TEXT, "Качество товара" "Качество");
-- Выдаём права пользователю u_0_00 SELECT, INSERT на таблицу "Товар"
GRANT SELECT, INSERT ON "Товар" TO u_0_00;
-- Отбираем права на использование типа 'Качество' у PUBLIC
REVOKE ALL ON TYPE "Качество" FROM PUBLIC;
-- Подключаемся как пользователь u_0_00
\connect - u_0_00
-- Проверяем возможность вставки в таблицу 'Товар'
-- Ожидается успех
INSERT INTO "Товар" VALUES ('Светильник', 'хорошее');
-- Ожидается успех
INSERT INTO "Товар" VALUES ('Лампочка накаливания', 'отличное');
-- Ожидается ошибка, т.к. запись 'не определено' не содержится в типе 'Качество'
INSERT INTO "Товар" VALUES ('Бра', 'не определено');
-- Убеждаемся в том, что в таблице 'Товар' были добавлены записи
-- Ожидается 2 записи
SELECT * FROM "Товар";
-- Проверяем возможность использования созданного типа в пользовательских таблицах
-- Ожидается ошибка, т.к. нет прав на использование типа "Качество" у пользователя u_0_00
CREATE TABLE "Качество работ"("Наименование" TEXT, "Оценка" "Качество");
--
\connect - postgres
-- Выдаем права на использование типа "Качество"
GRANT ALL ON TYPE "Качество" TO u_0_00;
--
\connect - u_0_00
-- Снова пытаемся создать таблицу
-- Ожидается успех
CREATE TABLE "Качество работ"("Наименование" TEXT, "Оценка" "Качество");
-- Проверяем возможность изменения типа
-- Ожидается ошибка
ALTER TYPE "Качество" ADD VALUE 'не определено';
--
\connect - postgres
-- Делаем пользователя владельцем типа
ALTER TYPE "Качество" OWNER TO u_0_00;
--
\connect - u_0_00
-- Повторяем проверку изменения типа
-- Ожидается успех
ALTER TYPE "Качество" ADD VALUE 'не определено';
-- Добавим строку в таблицу 'Товар'
-- Ожидается успех
INSERT INTO "Товар" VALUES ('Бра', 'не определено');
-- Убедимся в том, что новая запись была добавлена в таблицу
-- Ожидается 3 записи
SELECT * FROM "Товар";
--
\connect - postgres
-- Удаляем созданные таблицу и тип
DROP TABLE "Товар";
DROP TABLE "Качество работ";
DROP TYPE "Качество";
--
--
-- ========== Проверка домена (Domain) ==========
--
--
-- Создаём домен, обеспечивающий проверку телефонного номера
CREATE DOMAIN "Телефон" AS TEXT CONSTRAINT "Проверка" CHECK (VALUE ~ '^8\(\d{3}\)\d{3}\-\d{2}\-\d{2}$');
-- Создаём тестовую таблицу
CREATE TABLE "Клиенты" ("ФИО" text, "Номер телефона" "Телефон");
-- Отбираем привилегию USAGE на созданный домен
REVOKE ALL ON DOMAIN "Телефон" FROM PUBLIC;
-- Выдаем права пользователю u_0_00 SELECT, INSERT на таблицу "Клиенты"
GRANT INSERT, SELECT ON "Клиенты" TO u_0_00;
--
\connect - u_0_00
-- Выполняется попытка привести строку к типу домена
-- Ожидается ошибка
SELECT cast('84951234567' AS "Телефон");
-- Ожидается ошибка
SELECT cast('8(495)1234567' AS "Телефон");
--Ожидается успех
SELECT cast('8(495)-123-45-67' AS "Телефон");
-- Проверяем возможность привести строку к типу домена
-- Ожидается успех
SELECT cast('8(495)123-45-67' AS "Телефон");
-- Проверяем валидацию телефона при вставке
-- Ожидается ошибка, т.к. во второй колонке строка не соответствует записи в домене
INSERT INTO "Клиенты" VALUES('Иванов И.И.', '84951234567');
-- Ожидается успех
INSERT INTO "Клиенты" VALUES('Иванов И.И.', '8(495)123-45-67');
-- Проводится попытка просмотра содержимого таблицы
-- Ожидается 1 запись
SELECT * FROM "Клиенты";
-- Проверяем возможность использования домена при создании таблицы
-- Ожидается ошибка, т.к. у пользователя u_0_00 отсутствует привилегия USAGE
CREATE TABLE "Продавцы" ("ФИО" text, "Номер телефона" "Телефон");
--
\connect - postgres
-- Выдаём право использоватеть домен пользователю
GRANT ALL ON DOMAIN "Телефон" TO u_0_00;
--
\connect - u_0_00
-- Повторяем процесс создания таблицы 'Продавцы'
-- Ожидается успех
CREATE TABLE "Продавцы" ("ФИО" text, "Номер телефона" "Телефон");
-- Проверяем возможность модификацию домена пользователем
-- Ожидается ошибка, т.к. пользователь u_0_00 не является владельцем домена
ALTER DOMAIN "Телефон" ADD CHECK (VALUE ~ '^8\(\d{3}\)\d{3}\s\d{2}\s\d{2}$');
--
\connect - postgres
-- Подключаемся как postgres и меняем владельца на домен
ALTER DOMAIN "Телефон" OWNER TO u_0_00; 
--
\connect - u_0_00
-- Осуществляем попытку добавления ограничения на домен
-- Ожидается успех
ALTER DOMAIN "Телефон" ADD CONSTRAINT "Проверка2" CHECK (VALUE ~ '^8\(\d{3}\)\d{3}-\d{2}-\d{2}$' OR VALUE ~ '^8\(\d{3}\)\d{3}\s\d{2}\s\d{2}$');
-- Удаляем старое ограничение
ALTER DOMAIN "Телефон" DROP CONSTRAINT "Проверка";
-- Проверяем возможность встаки записи, которая будет содержать номер телефона с '-' и без
INSERT INTO "Клиенты" VALUES('Петров П.П.', '8(495)234 56 78');
INSERT INTO "Клиенты" VALUES('Васильев В.В.', '8(495)345-67-89');
-- Убеждаемся в том, что были добавлены записи пользователей 'Петров П.П.' и 'Васильев В.В'
-- Ожидается 3 записи
SELECT * FROM "Клиенты";
\connect - postgres
-- Удаляем созданные таблицы
DROP TABLE "Клиенты";
DROP TABLE "Продавцы";
-- Удаляем созданный домен
DROP DOMAIN "Телефон";
--
\i 'support/done.sql'
