-- RBT-TEST - ДПРД для видов
-- Проверка дискреционного разграничения доступа для видов
-- Права доступа для видов: SELECT, INSERT, UPDATE, DELETE
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
--
-- Общая подготовка
\connect - postgres
--
-- Отбираем права у PUBLIC на тестовую таблицу
REVOKE ALL ON "Проверка" FROM PUBLIC;
--
-- Создание проверочного вида
CREATE VIEW "Вид" AS SELECT * FROM "Проверка";
--
-- Сбрасываем метку у вида
ALTER TABLE "Вид" SET MAC TO NULL;
--
-- Создание соответствующих правил для вида
CREATE RULE "Вид_INSERT" AS
	ON INSERT TO "Вид"
		DO INSTEAD 
			INSERT INTO "Проверка" VALUES (new."идентификатор", new."данные");

CREATE RULE "Вид_UPDATE" AS
	ON UPDATE TO "Вид"
		DO INSTEAD 
			UPDATE "Проверка" SET "идентификатор" = new."идентификатор" , "данные" = new."данные"
				WHERE "идентификатор" = old."идентификатор";

CREATE RULE "Вид_DELETE" AS
	ON DELETE TO "Вид"
		DO INSTEAD DELETE FROM "Проверка" WHERE "идентификатор" = old."идентификатор";
--
-- Добавим несколько записей от лица владельца таблицы "Проверка"
INSERT INTO "Проверка" VALUES (1, 'Первая запись ' || USER); 
INSERT INTO "Проверка" VALUES (2, 'Вторая запись ' || USER);
--
-- Просматриваем созданные записи
SELECT * FROM "Проверка";
--
-- Просматриваем созданные записи
SELECT * FROM "Вид";
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Изначально все действия пользователю над видом "Вид" запрещены
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Вид";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Вид" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Вид" VALUES (3, 'Запись пользователя u_1_10');
--
-- Администратор postgres дает право SELECT для пользователя u_1_10
\connect - postgres
GRANT SELECT ON "Вид" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается успех
SELECT * FROM "Вид";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Вид" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Вид" VALUES (3, 'Запись пользователя u_1_10');
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции INSERT
-- и отбирает право на операцию SELECT
\connect - postgres
GRANT INSERT ON "Вид" TO u_1_10;
REVOKE SELECT ON "Вид" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Вид";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 2;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Вид" WHERE "идентификатор" = 2;
--
-- Попытка выполнить INSERT
-- Ожидается успех
INSERT INTO "Вид" VALUES (3, 'Запись пользователя u_1_10');
--
-- Переподключиться от имени администратора postgres
\connect - postgres
--
-- Убедимся, что в виде "Вид" появилась запись, занесенная пользователем u_1_10
SELECT * FROM "Вид";
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции UPDATE
-- и отбирает право на операцию INSERT
GRANT UPDATE ON "Вид" TO u_1_10;
REVOKE INSERT ON "Вид" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Вид";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа из-за отсутствия права SELECT
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа
DELETE FROM "Вид" WHERE "идентификатор" = 3;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Вид" VALUES (3, 'Запись пользователя u_1_10');
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции SELECT
\connect - postgres
GRANT SELECT ON "Вид" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить UPDATE
-- Ожидается успех, так как разрешены и SELECT и UPDATE
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 3;
--
-- В виде "Вид" идентификатор у записи пользователя увеличился на единицу
SELECT * FROM "Вид";
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции DELETE
-- и отбирает право на операцию UPDATE и SELECT
\connect - postgres
GRANT DELETE ON "Вид" TO u_1_10;
REVOKE UPDATE ON "Вид" FROM u_1_10;
REVOKE SELECT ON "Вид" FROM u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить SELECT
-- Ожидается ошибка доступа
SELECT * FROM "Вид";
--
-- Попытка выполнить UPDATE
-- Ожидается ошибка доступа
UPDATE "Вид" SET "идентификатор" = "идентификатор" + 1 WHERE "идентификатор" = 4;
--
-- Попытка выполнить DELETE
-- Ожидается ошибка доступа из-за отсутствия права SELECT
DELETE FROM "Вид" WHERE "идентификатор" = 4;
--
-- Попытка выполнить INSERT
-- Ожидается ошибка доступа
INSERT INTO "Вид" VALUES (3, 'Запись пользователя u_1_10');
--
-- Администратор postgres предоставляет пользователю u_1_10 право на выполнение операции SELECT
\connect - postgres
GRANT SELECT ON "Вид" TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка выполнить DELETE
-- Ожидается успех, так как разрешены и SELECT и DELETE
DELETE FROM "Вид" WHERE "идентификатор" = 4;
--
-- В виде "Вид" запись пользователя была удалена
SELECT * FROM "Вид";
--
\i 'support/done.sql'
