-- RBT-TEST - МПРД при модификации метаданных
-- Проверка работы МПРД при выполнении команд изменяющих схему
-- ALTER, DROP, COMMENT, TRUNCATE
-- при установленном параметре разрешения использования меток последовательностей ac_enable_sequence_mac
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Администратор postgres дает право CREATE на базу данных для пользователя u_1_11
\connect - postgres
GRANT CREATE ON DATABASE mtest TO u_1_11;
--
-- Создает роль и назначает ее пользователю u_1_11
CREATE ROLE "роль";
GRANT "роль" TO u_1_11;
--
-- Создает табличное пространство и дает права на его использование пользователю u_1_11
CREATE TABLESPACE dbspace LOCATION '/var/lib/postgresql/9.2/setest/testspace';
GRANT ALL ON TABLESPACE dbspace TO u_1_11;
GRANT ALL ON TABLESPACE pg_default TO u_1_11;
--
-- Проверим выполнение команд для таблиц
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_1_11 ("идентификатор" integer);
--
-- Создаем от его имени схему
CREATE SCHEMA "схема";
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы соответствует метке пользователя {1,3} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу со своей меткой
-- Ожидается успех
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 DROP CONSTRAINT idnpk;
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_1_11;
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_u_1_11 SET SCHEMA public;
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
ALTER TABLE test_u_1_11 SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_1_11;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."данные" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_1_11 DROP COLUMN "текст";
--
-- Администратор кластера postgres устанавливает метку таблицы в {0,0}
\connect - postgres
ALTER TABLE test_u_1_11 SET MAC TO '{0,0}';
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы установлена в {0,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою таблицу с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_1_11 DROP COLUMN "идентификатор";
--
-- Администратор кластера postgres устанавливает метку таблицы в {2,0}
\connect - postgres
ALTER TABLE test_u_1_11 SET MAC TO '{2,0}';
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы установлена в {2,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою таблицу с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_1_11 DROP COLUMN "идентификатор";
--
-- Администратор кластера postgres сбрасывает метку таблицы
\connect - postgres
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы сброшена и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу, если у нее сброшена метка
-- Ожидается успех
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 DROP CONSTRAINT idnpk;
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_1_11;
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_u_1_11 SET SCHEMA public;
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
ALTER TABLE test_u_1_11 SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_1_11;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_1_11."данные" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_1_11 DROP COLUMN "текст";
--
-- Администратор кластера postgres устанавливает метку таблицы в {2,0}
-- и устанавливает метку столбца "идентификатор" в {2,0}
-- включает режим использования меток столбцов
\connect - postgres
ALTER TABLE test_u_1_11 SET MAC TO '{2,0}';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET MAC TO '{2,0}';
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы установлена в {2,0} и режим использования меток столбцов включен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу
-- если используются метки столбцов
-- Ожидается успех
COMMENT ON TABLE test_u_1_11 IS 'проверочная таблица';
ALTER TABLE test_u_1_11 SET WITH OIDS;
ALTER TABLE test_u_1_11 ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_1_11 DROP CONSTRAINT idnpk;
ALTER TABLE test_u_1_11 ENABLE TRIGGER ALL;
ALTER TABLE test_u_1_11 SET WITHOUT CLUSTER;
ALTER TABLE test_u_1_11 OWNER TO "роль";
ALTER TABLE test_u_1_11 OWNER TO u_1_11;
--
-- Операции затрагивающие именование объекта или данные требуют наличия доступа ко всем столбцам
-- если используются метки столбцов
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 RENAME TO "таблица";
ALTER TABLE test_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_u_1_11 SET TABLESPACE dbspace;
TRUNCATE TABLE test_u_1_11;
--
-- пользователь добавляет столбец со своей меткой
ALTER TABLE test_u_1_11 ADD COLUMN "данные" TEXT;
--
-- Проверка меток столбцов таблицы test_u_1_11
-- Ожидается, что метка столбца "данные" соответствуют метке пользователя {1,3}
-- а метка столбца "идентификатор" установлена в {2,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attnum >= 0 and attisdropped = false;
--
-- Проверяем, что пользователь имеет возможность модифицировать столбцы с меткой, равной своей
-- Ожидается успех
COMMENT ON COLUMN test_u_1_11."данные" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_1_11 DROP COLUMN "текст";
--
-- модифицировать столбцы с меткой, отличной от своей не может
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_1_11 ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_1_11 DROP COLUMN "идентификатор";
--
--
-- Проверим выполнение команд для видов
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создание проверочного вида
CREATE VIEW test_view_u_1_11 AS SELECT * FROM test_u_1_11;
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида соответствует метке пользователя {1,3} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид со своей меткой
-- Ожидается успех
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 OWNER TO u_1_11;
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_1_11;
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
ALTER VIEW "схема".test_view_u_1_11 SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_1_11;
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_view_u_1_11 SET SCHEMA public;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идн" TO "идентификатор";
--
-- Администратор кластера postgres устанавливает метку вида в {0,0}
\connect - postgres
ALTER TABLE test_view_u_1_11 SET MAC TO '{0,0}';
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида установлена в {0,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свой вид с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
--
-- Администратор кластера postgres устанавливает метку вида в {2,0}
\connect - postgres
ALTER TABLE test_view_u_1_11 SET MAC TO '{2,0}';
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида установлена в {2,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свой вид с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
--
-- Администратор кластера postgres сбрасывает метку вида
\connect - postgres
ALTER TABLE test_view_u_1_11 SET MAC TO NULL;
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида сброшена и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид, если у него сброшена метка
-- Ожидается успех
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 OWNER TO u_1_11;
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_1_11;
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
ALTER VIEW "схема".test_view_u_1_11 SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 OWNER TO u_1_11;
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_1_11;
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_view_u_1_11 SET SCHEMA public;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идн" TO "идентификатор";
--
-- Администратор кластера postgres устанавливает метку вида в {2,0}
-- и устанавливает метку столбца "идентификатор" в {2,0}
-- включает режим использования меток столбцов
\connect - postgres
ALTER TABLE test_view_u_1_11 SET MAC TO '{2,0}';
ALTER TABLE test_view_u_1_11 ALTER COLUMN "идентификатор" SET MAC TO '{2,0}';
ALTER TABLE test_view_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка метки вида test_view_u_1_11
-- Ожидается, что метка вида установлена в {2,0} и режим использования меток столбцов включен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_1_11'::regclass::oid;
--
-- Проверка метки столбцов вида test_view_u_1_11
-- Ожидается, что метка столбца "идентификатор" установлена в {2,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_view_u_1_11'::regclass::oid and attnum >= 0 and attisdropped = false;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид
-- если используются метки столбцов
-- Ожидается успех
COMMENT ON VIEW test_view_u_1_11 IS 'проверочный вид';
ALTER VIEW test_view_u_1_11 OWNER TO "роль";
ALTER VIEW test_view_u_1_11 OWNER TO u_1_11;
--
-- Операции затрагивающие именование объекта или данные требуют наличия доступа ко всем столбцам
-- если используются метки столбцов
-- Ожидается ошибка доступа
ALTER VIEW test_view_u_1_11 RENAME TO "вид";
ALTER VIEW test_view_u_1_11 SET SCHEMA "схема";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 OWNER TO "роль";
ALTER TABLE test_view_u_1_11 OWNER TO u_1_11;
--
-- Операции затрагивающие именование объекта или данные требуют наличия доступа ко всем столбцам
-- если используются метки столбцов
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME TO "вид";
ALTER TABLE test_view_u_1_11 SET SCHEMA "схема";
--
-- модифицировать столбцы с меткой, отличной от своей не может
-- Ожидается ошибка доступа
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
--
-- Администратор кластера postgres устанавливает метку столбца вида в {1,3}
\connect - postgres
ALTER TABLE test_view_u_1_11 ALTER COLUMN "идентификатор" SET MAC TO '{1,3}';
--
-- Проверка метки столбцов вида test_view_u_1_11
-- Ожидается, что метка столбца "идентификатор" установлена в {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_view_u_1_11'::regclass::oid and attnum >= 0 and attisdropped = false;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь может модифицировать столбцы с меткой равной своей
-- Ожидается успех
COMMENT ON COLUMN test_view_u_1_11."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_1_11 ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_1_11 RENAME COLUMN "идн" TO "идентификатор";
--
--
-- Проверим выполнение команд ALTER SEQUENCE и ALTER TABLE для последовательностей
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Создаем проверочную последовательность
CREATE SEQUENCE test_seq_u_1_11
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности соответствует метке пользователя {1,3}
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность со своей меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_1_11;
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_1_11;
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
ALTER TABLE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Администратор кластера postgres устанавливает метку последовательности в {0,0}
\connect - postgres
ALTER TABLE test_seq_u_1_11 SET MAC TO '{0,0}';
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности установлена в {0,0}
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою последовательность с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
--
-- Администратор кластера postgres устанавливает метку последовательности в {2,0}
\connect - postgres
ALTER TABLE test_seq_u_1_11 SET MAC TO '{2,0}';
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности установлена в {2,0}
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность модифицировать свою последовательность с неравной своей меткой
-- Ожидается ошибка доступа
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается ошибка доступа
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
--
-- Администратор кластера postgres сбрасывает метку последовательности
\connect - postgres
ALTER TABLE test_seq_u_1_11 SET MAC TO NULL;
--
-- Проверка метки последовательности test_seq_u_1_11
-- Ожидается, что метка последовательности сброшена
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_1_11'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность, если у нее сброшена метка
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_1_11 IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_1_11 INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_1_11 RESTART;
ALTER SEQUENCE test_seq_u_1_11 RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_1_11;
ALTER SEQUENCE test_seq_u_1_11 SET SCHEMA "схема";
ALTER SEQUENCE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER SEQUENCE test_seq_u_1_11 OWNER TO "роль";
ALTER SEQUENCE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_1_11 RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_1_11;
ALTER TABLE test_seq_u_1_11 SET SCHEMA "схема";
ALTER TABLE "схема".test_seq_u_1_11 SET SCHEMA public;
ALTER TABLE test_seq_u_1_11 OWNER TO "роль";
ALTER TABLE test_seq_u_1_11 OWNER TO u_1_11;
--
-- Проверим выполнение команды DROP для объектов
-- DROP для столбцов уже был рассмотрен выше
--
-- Администратор кластера postgres отключает режим использования меток столбцов
\connect - postgres
ALTER TABLE test_u_1_11 DISABLE COLUMN MACS;
ALTER TABLE test_view_u_1_11 DISABLE COLUMN MACS;
ALTER TABLE test_seq_u_1_11 SET MAC TO '{2,0}';
--
-- Проверка метки объектов
-- Ожидается, что метки установлены в {2,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE relname LIKE 'test_%';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность удалять свои объекты с отличной от своей меткой
-- Ожидается ошибка доступа
DROP SEQUENCE test_seq_u_1_11;
DROP VIEW test_view_u_1_11;
DROP TABLE test_u_1_11;
--
-- Администратор кластера postgres устанавливает метки объектов в {1, 3}
-- а метки столбцов в {2,0}
-- и включает режим использования меток столбцов
\connect - postgres
ALTER TABLE test_seq_u_1_11 SET MAC TO '{1,3}';
ALTER TABLE test_view_u_1_11 SET MAC TO '{1,3}';
ALTER TABLE test_u_1_11 SET MAC TO '{1,3}';
ALTER TABLE test_view_u_1_11 ALTER COLUMN "идентификатор" SET MAC TO '{2,0}';
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
ALTER TABLE test_view_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка метки объектов
-- Ожидается, что метки установлены в {1,3} и режим использования меток столбцов включен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE relname LIKE 'test_%';
--
-- Проверка метки столбцов объектов
-- Ожидается, что метка столбца "идентификатор" установлена в {2,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE (attrelid = 'test_u_1_11'::regclass::oid or attrelid = 'test_view_u_1_11'::regclass::oid) and attnum >= 0 and attisdropped = false;
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь не имеет возможность удалять свои объекты
-- если есть столбцы с отличной от своей меткой
-- Ожидается ошибка доступа
DROP VIEW test_view_u_1_11;
DROP TABLE test_u_1_11;
--
-- Администратор кластера postgres отключает режим использования меток столбцов
\connect - postgres
ALTER TABLE test_u_1_11 DISABLE COLUMN MACS;
ALTER TABLE test_view_u_1_11 DISABLE COLUMN MACS;
--
-- Проверка метки объектов
-- Ожидается, что метки установлены в {1,3} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE relname LIKE 'test_%';
--
-- Подключаемся как пользователь u_1_11 с меткой {1,3}
\connect - u_1_11
--
-- Проверяем, что пользователь может удалять свои объекты со своей меткой
-- Ожидается успех
DROP SEQUENCE test_seq_u_1_11;
DROP VIEW test_view_u_1_11;
DROP TABLE test_u_1_11;
--
-- 
-- Проверка доступности всех команд для пользователя с атрибутами игнорирования мандатного доступа dbsa
--
-- Администратор postgres дает право CREATE на базу данных для пользователя dbsa
\connect - postgres
GRANT CREATE ON DATABASE mtest TO dbsa;
--
-- Назначает роль пользователю dbsa
GRANT "роль" TO dbsa;
--
-- Дает права на использование области хранения данных пользователю dbsa
GRANT ALL ON TABLESPACE dbspace TO dbsa;
GRANT ALL ON TABLESPACE pg_default TO dbsa;
--
-- Подключаемся как пользователь dbsa с меткой {2,3}
\connect - dbsa
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_dbsa ("идентификатор" integer);
--
-- Создаем от его имени схему
CREATE SCHEMA "схема dbsa";
--
-- Проверка метки таблицы test_u_dbsa
-- Ожидается, что метка таблицы соответствует метке пользователя {2,3} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу со своей меткой
-- Ожидается успех
COMMENT ON TABLE test_u_dbsa IS 'проверочная таблица';
ALTER TABLE test_u_dbsa SET WITH OIDS;
ALTER TABLE test_u_dbsa ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_dbsa DROP CONSTRAINT idnpk;
ALTER TABLE test_u_dbsa ENABLE TRIGGER ALL;
ALTER TABLE test_u_dbsa SET WITHOUT CLUSTER;
ALTER TABLE test_u_dbsa OWNER TO "роль";
ALTER TABLE test_u_dbsa OWNER TO dbsa;
ALTER TABLE test_u_dbsa RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_dbsa;
ALTER TABLE test_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_u_dbsa SET SCHEMA public;
ALTER TABLE test_u_dbsa SET TABLESPACE dbspace;
ALTER TABLE test_u_dbsa SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_dbsa;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
ALTER TABLE test_u_dbsa ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_dbsa."данные" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_dbsa DROP COLUMN "текст";
--
-- Пользователь dbsa устанавливает метку таблицы в {0,0}
ALTER TABLE test_u_dbsa SET MAC TO '{0,0}';
--
-- Проверка метки таблицы test_u_dbsa
-- Ожидается, что метка таблицы установлена в {0,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу с любой меткой
-- Ожидается успех
COMMENT ON TABLE test_u_dbsa IS 'проверочная таблица';
ALTER TABLE test_u_dbsa SET WITH OIDS;
ALTER TABLE test_u_dbsa ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_dbsa DROP CONSTRAINT idnpk;
ALTER TABLE test_u_dbsa ENABLE TRIGGER ALL;
ALTER TABLE test_u_dbsa SET WITHOUT CLUSTER;
ALTER TABLE test_u_dbsa OWNER TO "роль";
ALTER TABLE test_u_dbsa OWNER TO dbsa;
ALTER TABLE test_u_dbsa RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_dbsa;
ALTER TABLE test_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_u_dbsa SET SCHEMA public;
ALTER TABLE test_u_dbsa SET TABLESPACE dbspace;
ALTER TABLE test_u_dbsa SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_dbsa;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
ALTER TABLE test_u_dbsa ADD COLUMN "данные" TEXT;
COMMENT ON COLUMN test_u_dbsa."данные" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_dbsa DROP COLUMN "текст";
--
-- Пользователь dbsa устанавливает столбца "идентификатор" в {0,0}
-- и включает режим использования меток столбцов
ALTER TABLE test_u_dbsa ALTER COLUMN "идентификатор" SET MAC TO '{0,0}';
ALTER TABLE test_u_dbsa ENABLE COLUMN MACS;
--
-- Проверка метки таблицы test_u_dbsa
-- Ожидается, что метка таблицы установлена в {2,3} и режим использования меток столбцов включен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою таблицу
-- если используются метки столбцов
-- Ожидается успех
COMMENT ON TABLE test_u_dbsa IS 'проверочная таблица';
ALTER TABLE test_u_dbsa SET WITH OIDS;
ALTER TABLE test_u_dbsa ADD CONSTRAINT idnpk PRIMARY KEY ("идентификатор");
ALTER TABLE test_u_dbsa DROP CONSTRAINT idnpk;
ALTER TABLE test_u_dbsa ENABLE TRIGGER ALL;
ALTER TABLE test_u_dbsa SET WITHOUT CLUSTER;
ALTER TABLE test_u_dbsa OWNER TO "роль";
ALTER TABLE test_u_dbsa OWNER TO dbsa;
ALTER TABLE test_u_dbsa RENAME TO "таблица";
ALTER TABLE "таблица" RENAME TO test_u_dbsa;
ALTER TABLE test_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_u_dbsa SET SCHEMA public;
ALTER TABLE test_u_dbsa SET TABLESPACE dbspace;
ALTER TABLE test_u_dbsa SET TABLESPACE pg_default;
TRUNCATE TABLE test_u_dbsa;
--
-- при включенном режиме использования меток столбцов может модифицировать любые столбцы
-- Ожидается успех
ALTER TABLE test_u_dbsa ADD COLUMN "данные" TEXT;
--
-- Проверка метки столбцов таблицы test_u_dbsa
-- Ожидается, что метка столбца "данные" соответствуют метке пользователя {2,3}
-- а метка столбца "идентификатор" установлена в {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_dbsa'::regclass::oid and attnum >= 0 and attisdropped = false;
--
-- ожидается успех для всех столбцов
COMMENT ON COLUMN test_u_dbsa."данные" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET DEFAULT 'пусто';
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "данные" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "данные" TO "текст";
ALTER TABLE test_u_dbsa DROP COLUMN "текст";
COMMENT ON COLUMN test_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER TABLE test_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER TABLE test_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
ALTER TABLE test_u_dbsa ALTER COLUMN "идентификатор" SET NOT NULL;
ALTER TABLE test_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_u_dbsa DROP COLUMN "идн";
ALTER TABLE test_u_dbsa ADD COLUMN "идентификатор" integer;
--
-- Проверим выполнение команд для видов
--
-- Создание проверочного вида
CREATE VIEW test_view_u_dbsa AS SELECT * FROM test_u_dbsa;
--
-- Проверка метки вида test_view_u_dbsa
-- Ожидается, что метка вида соответствует метке пользователя {2,3} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид со своей меткой
-- Ожидается успех
COMMENT ON VIEW test_view_u_dbsa IS 'проверочный вид';
ALTER VIEW test_view_u_dbsa OWNER TO "роль";
ALTER VIEW test_view_u_dbsa OWNER TO dbsa;
ALTER VIEW test_view_u_dbsa RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_dbsa;
ALTER VIEW test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER VIEW "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa OWNER TO "роль";
ALTER TABLE test_view_u_dbsa OWNER TO dbsa;
ALTER TABLE test_view_u_dbsa RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_dbsa;
ALTER TABLE test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
COMMENT ON COLUMN test_view_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идн" TO "идентификатор";
--
-- Пользователь dbsa устанавливает метку вида в {0,0}
ALTER TABLE test_view_u_dbsa SET MAC TO '{0,0}';
--
-- Проверка метки вида test_view_u_dbsa
-- Ожидается, что метка вида установлена в {0,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид с неравной своей меткой
-- Ожидается успех
COMMENT ON VIEW test_view_u_dbsa IS 'проверочный вид';
ALTER VIEW test_view_u_dbsa OWNER TO "роль";
ALTER VIEW test_view_u_dbsa OWNER TO dbsa;
ALTER VIEW test_view_u_dbsa RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_dbsa;
ALTER VIEW test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER VIEW "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa OWNER TO "роль";
ALTER TABLE test_view_u_dbsa OWNER TO dbsa;
ALTER TABLE test_view_u_dbsa RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_dbsa;
ALTER TABLE test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- при отключенном режиме использования меток столбцов при модификации столбцов применяется метка объекта
-- Ожидается успех
COMMENT ON COLUMN test_view_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идн" TO "идентификатор";
--
-- Пользователь dbsa устанавливает метку вида в {2,3}
-- а метку  столбца "идентификатор" в {0,0}
-- включает режим использования меток столбцов
ALTER TABLE test_view_u_dbsa SET MAC TO '{2,3}';
ALTER TABLE test_view_u_dbsa ALTER COLUMN "идентификатор" SET MAC TO '{0,0}';
ALTER TABLE test_view_u_dbsa ENABLE COLUMN MACS;
--
-- Проверка метки вида test_view_u_dbsa
-- Ожидается, что метка вида установлена в {0,0} и режим использования меток столбцов включен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE oid = 'test_view_u_dbsa'::regclass::oid;
--
-- Проверка метки столбцов вида test_view_u_1_11
-- Ожидается, что метка столбца "идентификатор" установлена в {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_view_u_dbsa'::regclass::oid and attnum >= 0 and attisdropped = false;
--
-- Проверяем, что пользователь имеет возможность модифицировать свой вид
-- если используются метки столбцов
COMMENT ON VIEW test_view_u_dbsa IS 'проверочный вид';
ALTER VIEW test_view_u_dbsa OWNER TO "роль";
ALTER VIEW test_view_u_dbsa OWNER TO dbsa;
ALTER VIEW test_view_u_dbsa RENAME TO "вид";
ALTER VIEW "вид" RENAME TO test_view_u_dbsa;
ALTER VIEW test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER VIEW "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa OWNER TO "роль";
ALTER TABLE test_view_u_dbsa OWNER TO dbsa;
ALTER TABLE test_view_u_dbsa RENAME TO "вид";
ALTER TABLE "вид" RENAME TO test_view_u_dbsa;
ALTER TABLE test_view_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_view_u_dbsa SET SCHEMA public;
--
-- при включенном режиме использования меток столбцов может модифицировать столбцы
-- Ожидается успех
COMMENT ON COLUMN test_view_u_dbsa."идентификатор" IS 'проверочный столбец';
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" SET DEFAULT 0;
ALTER VIEW test_view_u_dbsa ALTER COLUMN "идентификатор" DROP DEFAULT;
-- 
-- Переименование столбцов командой ALTER TABLE
-- Ожидается успех
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идентификатор" TO "идн";
ALTER TABLE test_view_u_dbsa RENAME COLUMN "идн" TO "идентификатор";
--
--
-- Проверим выполнение команд ALTER SEQUENCE и ALTER TABLE для последовательностей
--
-- Создаем проверочную последовательность
CREATE SEQUENCE test_seq_u_dbsa
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
--
-- Проверка метки последовательности test_seq_u_dbsa
-- Ожидается, что метка последовательности соответствует метке пользователя {2,3}
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность со своей меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_dbsa IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_dbsa INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_dbsa RESTART;
ALTER SEQUENCE test_seq_u_dbsa RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER SEQUENCE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER SEQUENCE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER SEQUENCE test_seq_u_dbsa OWNER TO "роль";
ALTER SEQUENCE test_seq_u_dbsa OWNER TO dbsa;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_dbsa RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER TABLE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER TABLE test_seq_u_dbsa OWNER TO "роль";
ALTER TABLE test_seq_u_dbsa OWNER TO dbsa;
--
-- Пользователь dbsa устанавливает метку последовательности в {0,0}
ALTER TABLE test_seq_u_dbsa SET MAC TO '{0,0}';
--
-- Проверка метки последовательности test_seq_u_dbsa
-- Ожидается, что метка последовательности установлена в {0,0}
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка"
FROM pg_class WHERE oid = 'test_seq_u_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь имеет возможность модифицировать свою последовательность с любой меткой
-- Ожидается успех
COMMENT ON SEQUENCE test_seq_u_dbsa IS 'проверочная последовательность';
ALTER SEQUENCE test_seq_u_dbsa INCREMENT BY 2;
ALTER SEQUENCE test_seq_u_dbsa RESTART;
ALTER SEQUENCE test_seq_u_dbsa RENAME TO "последовательность";
ALTER SEQUENCE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER SEQUENCE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER SEQUENCE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER SEQUENCE test_seq_u_dbsa OWNER TO "роль";
ALTER SEQUENCE test_seq_u_dbsa OWNER TO dbsa;
--
-- Аналогично используются команды ALTER TABLE
-- Ожидается успех
ALTER TABLE test_seq_u_dbsa RENAME TO "последовательность";
ALTER TABLE "последовательность" RENAME TO test_seq_u_dbsa;
ALTER TABLE test_seq_u_dbsa SET SCHEMA "схема dbsa";
ALTER TABLE "схема dbsa".test_seq_u_dbsa SET SCHEMA public;
ALTER TABLE test_seq_u_dbsa OWNER TO "роль";
ALTER TABLE test_seq_u_dbsa OWNER TO dbsa;
--
-- Пользователь dbsa отключает режим использования меток столбцов
ALTER TABLE test_u_dbsa DISABLE COLUMN MACS;
ALTER TABLE test_view_u_dbsa DISABLE COLUMN MACS;
ALTER TABLE test_seq_u_dbsa SET MAC TO '{0,0}';
ALTER TABLE test_view_u_dbsa SET MAC TO '{0,0}';
--
-- Проверка метки объектов
-- Ожидается, что метки установлены в {0,0} и режим использования меток столбцов отключен
SELECT relname AS "Наименование объекта", relmaclabel AS "Метка", relusecolmacs AS "Использование меток столбцов"
FROM pg_class WHERE relname LIKE 'test_%';
--
-- Проверяем, что пользователь имеет возможность удалять свои объекты с любой меткой
-- Ожидается успех
DROP SEQUENCE test_seq_u_dbsa;
DROP VIEW test_view_u_dbsa;
DROP TABLE test_u_dbsa;
--
-- Администратор кластера postgres удаляет созданные объекты
\connect - postgres
REVOKE "роль" FROM u_1_11;
REVOKE "роль" FROM dbsa;
DROP ROLE "роль";
REVOKE ALL ON TABLESPACE pg_default FROM u_1_11;
REVOKE ALL ON TABLESPACE pg_default FROM dbsa;
DROP TABLESPACE dbspace;
--
\i 'support/done.sql'
