-- RBT-TEST - ALTER TABLE ? ENABLE COLUMN MACS
-- Проверка работы ALTER TABLE ... ENABLE/DISABLE COLUMN MACS
-- при установленном параметре разрешения использования меток последовательностей ac_enable_sequence_mac
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Проверим выполнение команды ALTER TABLE ... ENABLE/DISABLE COLUMN MACS для таблиц
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_0_00 (data TEXT);
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять признак использования меток столбцов
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем таблицу от его имени
CREATE TABLE test_u_1_11 (data TEXT);
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять признак использования меток столбцов
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователи не имеют возможности поменять признаки использования меток столбцов таблиц друг у друга
--
-- Администратор сбрасывает метки таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 SET MAC TO NULL;
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается изменить признак использования меток столбцов таблицы пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять признаки использования меток столбцов таблиц друг у друга
-- Пользователь u_0_00 пытается изменить признак использования меток столбцов таблицы пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор возвращает метки таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 SET MAC TO '{0,0}';
ALTER TABLE test_u_1_11 SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем таблицу от его имени
CREATE TABLE test_dbsa (data TEXT);
--
-- Проверка признака использования меток столбцов таблицы test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что он может менять признак использования меток столбцов своей таблицы
ALTER TABLE test_dbsa ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_dbsa
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что он может сбросить признак использования меток столбцов своей таблицы
ALTER TABLE test_dbsa DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь dbsa с флагом CAP_CHMAC не имеет возможности поменять
-- признаки использования меток столбцов таблиц других пользователей
-- Пользователь dbsa пытается изменить признак использования меток столбцов таблицы пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Пользователь dbsa пытается изменить признак использования меток столбцов таблицы пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что администратор может менять признаки использования меток столбцов любых таблиц
\connect - postgres
--
-- Администратор устанавливает признак использования меток столбцов таблицы пользователя u_0_00
ALTER TABLE test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Администратор устанавливает признак использования меток столбцов таблицы пользователя u_1_11
ALTER TABLE test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор устанавливает признак использования меток столбцов таблицы пользователя dbsa
ALTER TABLE test_dbsa ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_dbsa
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов таблицы пользователя u_0_00
ALTER TABLE test_u_0_00 DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов таблицы пользователя u_1_11
ALTER TABLE test_u_1_11 DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов таблицы пользователя dbsa
ALTER TABLE test_dbsa DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов таблицы test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование таблицы", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Попытка установить признак использования меток столбцов последовательности
-- Ожидается ошибка доступа
CREATE SEQUENCE test_seq;
ALTER TABLE test_seq ENABLE COLUMN MACS;
--
-- Попытка установить признак использования меток столбцов индекса
-- Ожидается ошибка доступа
CREATE INDEX test_idx ON test_u_0_00(data);
ALTER TABLE test_idx ENABLE COLUMN MACS;
--
-- Проверяем, что признак использования меток столбцов у системных таблиц изменить нельзя
-- Попытки установки признака использования меток столбцов системных таблиц
-- Ожидается ошибка доступа
ALTER TABLE pg_catalog.pg_class ENABLE COLUMN MACS;
ALTER TABLE information_schema.sql_features ENABLE COLUMN MACS;
--
-- Проверка выполнения команды ALTER TABLE ... ENABLE/DISABLE COLUMN MACS для видов
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени вид
CREATE VIEW view_test_u_0_00 AS SELECT * from pg_user;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять признак использования меток столбцов
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем вид от его имени
CREATE VIEW view_test_u_1_11 AS SELECT * from pg_user;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять признак использования меток столбцов
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователи не имеют возможности поменять признаки использования меток столбцов видов друг у друга
--
-- Администратор сбрасывает метки видов view_test_u_0_00 и view_test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 SET MAC TO NULL;
ALTER TABLE view_test_u_1_11 SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается изменить признак использования меток столбцов вида пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять признаки использования меток столбцов видов друг у друга
-- Пользователь u_0_00 пытается изменить признак использования меток столбцов вида пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор возвращает метки таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 SET MAC TO '{0,0}';
ALTER TABLE view_test_u_1_11 SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем вид от его имени
CREATE VIEW view_test_dbsa AS SELECT * from pg_user;
--
-- Проверка признака использования меток столбцов вида view_test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC может менять признак использования меток столбцов своего вида
ALTER TABLE view_test_dbsa ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_dbsa
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC может сбросить признак использования меток столбцов своего вида
ALTER TABLE view_test_dbsa DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Но не может менять признак использования меток столбцов видов других пользователей
-- Пользователь dbsa пытается изменить признак использования меток столбцов вида пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Пользователь dbsa пытается изменить признак использования меток столбцов вида пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, что администратор может менять признаки использования меток столбцов у любых видов
\connect - postgres
--
-- Администратор устанавливает признак использования меток столбцов вида пользователя u_0_00
ALTER TABLE view_test_u_0_00 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов вида пользователя u_0_00
ALTER TABLE view_test_u_0_00 DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_0_00
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Администратор устанавливает признак использования меток столбцов вида пользователя u_1_11
ALTER TABLE view_test_u_1_11 ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов вида пользователя u_1_11
ALTER TABLE view_test_u_1_11 DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_u_1_11
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор устанавливает признак использования меток столбцов вида пользователя dbsa
ALTER TABLE view_test_dbsa ENABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_dbsa
-- Ожидается, что признак использования меток столбцов установлен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Администратор сбрасывает признак использования меток столбцов вида пользователя dbsa
ALTER TABLE view_test_dbsa DISABLE COLUMN MACS;
--
-- Проверка признака использования меток столбцов вида view_test_dbsa
-- Ожидается, что признак использования меток столбцов сброшен
SELECT relname AS "Наименование вида", relusecolmacs AS "Признак использования меток столбцов"
FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
\i 'support/done.sql'
