-- RBT-TEST - ALTER TABLE ? ALTER COLUMN ? SET MAC TO ?
-- Проверка работы ALTER TABLE ... ALTER COLUMN ... SET MAC TO ...
-- при установленном параметре разрешения использования меток последовательностей ac_enable_sequence_mac
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Проверим выполнение команды ALTER TABLE ... ALTER COLUMN ... SET MAC TO ... для таблиц
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_0_00 (data TEXT);
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца соответствует метке пользователя {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку столбца таблицы
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца осталась той же {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем таблицу от его имени
CREATE TABLE test_u_1_11 (data TEXT);
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца соответствует метке пользователя {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку столбца таблицы
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца осталась той же {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Проверяем, что пользователи не имеют возможности поменять метки столбцов таблиц друг у друга
--
-- Администратор сбрасывает метку столбцов таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO NULL;
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается установить метку на столбец таблицы пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO '{1,3}';
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца осталась сброшенной
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять метки столбцов таблиц друг у друга
-- Пользователь u_0_00 пытается установить метку на столбец таблицы пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO '{0,0}';
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца осталась сброшенной
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Администратор возвращает метки столбцов таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO '{0,0}';
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем таблицу от его имени
CREATE TABLE test_dbsa (data TEXT);
--
-- Проверка метки столбца data таблицы test_dbsa
-- Ожидается, что метка столбца соответствует метке пользователя {2,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_dbsa'::regclass::oid and attname = 'data';
--
-- Проверяем, что он может убрать метку столбца своей таблицы
ALTER TABLE test_dbsa ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_dbsa
-- Ожидается, что столбец не имеет метки
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_dbsa'::regclass::oid and attname = 'data';
--
-- Проверяем, что он может может сменить метку столбца своей таблицы на {1,1}
ALTER TABLE test_dbsa ALTER COLUMN data SET MAC TO '{1,1}';
--
-- Проверка метки столбца data таблицы test_dbsa
-- Ожидается, что метка столбца равна {1,1}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_dbsa'::regclass::oid and attname = 'data';
--
-- Проверяем, что пользователь dbsa с флагом CAP_CHMAC не имеет возможности поменять метки
-- столбцов таблиц других пользователей
-- Пользователь dbsa пытается сбросить метку у столбца таблицы пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца осталась той же {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Пользователь dbsa пытается сбросить метку у столбца таблицы пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца осталась той же {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Проверяем, что администратор может менять метки у столбцов любых таблиц
\connect - postgres
--
-- Администратор сбрасывает метку у столбца таблицы пользователя u_0_00
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Администратор устанавливает метку столбца таблицы пользователя u_0_00
ALTER TABLE test_u_0_00 ALTER COLUMN data SET MAC TO '{0,0}';
--
-- Проверка метки столбца data таблицы test_u_0_00
-- Ожидается, что метка столбца установлена в {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_0_00'::regclass::oid and attname = 'data';
--
-- Администратор сбрасывает метку у столбца таблицы пользователя u_1_11
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Администратор устанавливает метку столбца таблицы пользователя u_1_11
ALTER TABLE test_u_1_11 ALTER COLUMN data SET MAC TO '{1,3}';
--
-- Проверка метки столбца data таблицы test_u_1_11
-- Ожидается, что метка столбца установлена в {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_u_1_11'::regclass::oid and attname = 'data';
--
-- Администратор сбрасывает метку у столбца таблицы пользователя dbsa
ALTER TABLE test_dbsa ALTER COLUMN data SET MAC TO NULL;
--
-- Проверка метки столбца data таблицы dbsa
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_dbsa'::regclass::oid and attname = 'data';
--
-- Администратор устанавливает метку столбца таблицы пользователя dbsa
ALTER TABLE test_dbsa ALTER COLUMN data SET MAC TO '{1,1}';
--
-- Проверка метки столбца data таблицы test_dbsa
-- Ожидается, что метка столбца равна {1,1}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'test_dbsa'::regclass::oid and attname = 'data';
--
-- Попытка установить метку столбца последовательности
-- Ожидается ошибка доступа
CREATE SEQUENCE test_seq;
ALTER TABLE test_seq ALTER COLUMN last_value SET MAC TO '{0,0}';
--
-- Попытка установить метку столбца индекса
-- Ожидается ошибка доступа
CREATE INDEX test_idx ON test_u_0_00(data);
ALTER TABLE test_idx ALTER COLUMN data SET MAC TO '{0,0}';
--
-- Проверяем, что метки у столбцов системных таблиц изменить нельзя
-- Попытки установки метки столбцам системных таблиц
-- Ожидается ошибка доступа
ALTER TABLE pg_catalog.pg_class ALTER COLUMN relname SET MAC TO '{0,0}';
ALTER TABLE information_schema.sql_features ALTER COLUMN feature_name SET MAC TO '{0,0}';
--
-- Проверка выполнения команды ALTER TABLE ... ALTER COLUMN ... SET MAC TO ... для видов
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени вид
CREATE VIEW view_test_u_0_00 AS SELECT * from pg_user;
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что столбец приобрел метку пользователя {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку столбца вида
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что метка столбца не изменилась {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем вид от его имени
CREATE VIEW view_test_u_1_11 AS SELECT * from pg_user;
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что столбец приобрел метку пользователя {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Проверяем, чтобы он так же не мог изменить метку столбца своего вида
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что метка столбца не изменилась {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Проверяем, что пользователи не имеют возможности поменять метки столбцов видов друг у друга
--
-- Администратор сбрасывает метку столбцов видов view_test_u_0_00 и view_test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO NULL;
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается установить метку на столбец вида пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO '{1,3}';
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что метка столбца осталась сброшенной
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять метки столбцов видов друг у друга
-- Пользователь u_0_00 пытается установить метку на столбец вида пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO '{0,0}';
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что метка столбца осталась сброшенной
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Администратор возвращает метки столбцов видов view_test_u_0_00 и view_test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO '{0,0}';
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем вид от его имени
CREATE VIEW view_test_dbsa AS SELECT * from pg_user;
--
-- Проверка метки столбца usename вида view_test_dbsa
-- Ожидается, что столбец приобрел метку пользователя {2,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_dbsa'::regclass::oid and attname = 'usename';
--
-- Проверяем, чтобы он может сбросить метку столбца своего вида
ALTER TABLE view_test_dbsa ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_dbsa
-- Ожидается, что метка столбца не установлена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_dbsa'::regclass::oid and attname = 'usename';
--
-- Проверяем, чтобы он может изменить метку столбца своего вида
ALTER TABLE view_test_dbsa ALTER COLUMN usename SET MAC TO '{1,1}';
--
-- Проверка метки столбца usename вида view_test_dbsa
-- Ожидается, что метка столбца установлена в {1,1}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_dbsa'::regclass::oid and attname = 'usename';
--
-- Но не может менять метки столбцов видов других пользователей
-- Пользователь dbsa пытается сбросить метку у столбца вида пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что метка столбца не изменилась {0,0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Пользователь dbsa пытается сбросить метку у столбца вида пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что метка столбца не изменилась {1,3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Проверяем, что администратор может менять метки у столбцов любых видов
\connect - postgres
--
-- Администратор сбрасывает метку у столбца вида пользователя u_0_00
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Администратор устанавливает метку у столбца вида пользователя u_0_00
ALTER TABLE view_test_u_0_00 ALTER COLUMN usename SET MAC TO '{0,0}';
--
-- Проверка метки столбца usename вида view_test_u_0_00
-- Ожидается, что метка столбца установлена в {0, 0}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_0_00'::regclass::oid and attname = 'usename';
--
-- Администратор сбрасывает метку у столбца вида пользователя u_1_11
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Администратор устанавливает метку у столбца вида пользователя u_1_11
ALTER TABLE view_test_u_1_11 ALTER COLUMN usename SET MAC TO '{1,3}';
--
-- Проверка метки столбца usename вида view_test_u_1_11
-- Ожидается, что метка столбца установлена в {1, 3}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_u_1_11'::regclass::oid and attname = 'usename';
--
-- Администратор сбрасывает метку у вида пользователя dbsa
ALTER TABLE view_test_dbsa ALTER COLUMN usename SET MAC TO NULL;
--
-- Проверка метки столбца usename вида view_test_dbsa
-- Ожидается, что метка столбца сброшена
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_dbsa'::regclass::oid and attname = 'usename';
--
-- Администратор устанавливает метку у вида пользователя dbsa
ALTER TABLE view_test_dbsa ALTER COLUMN usename SET MAC TO '{1,1}';
--
-- Проверка метки столбца usename вида view_test_dbsa
-- Ожидается, что метка столбца установлена в {1,1}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'view_test_dbsa'::regclass::oid and attname = 'usename';
--
\i 'support/done.sql'
