-- RBT-TEST - ALTER TABLE ? SET MAC TO ?
-- Проверка работы ALTER TABLE ... SET MAC TO ...
-- при установленном параметре разрешения использования меток последовательностей ac_enable_sequence_mac
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Проверим выполнение команды ALTER TABLE ... SET MAC TO ... для таблиц
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени таблицу
CREATE TABLE test_u_0_00 (data TEXT);
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что метка таблицы соответствует метке пользователя {0,0}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку таблицы
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что метка таблицы осталась той же {0,0}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем таблицу от его имени
CREATE TABLE test_u_1_11 (data TEXT);
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы соответствует метке пользователя {1,3}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что он так же не может изменить метку своей таблицы
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы осталась той же {1,3}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователи не имеют возможности поменять метки таблиц друг у друга
--
-- Администратор сбрасывает метку таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 SET MAC TO NULL;
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается установить метку на таблицу пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 SET MAC TO '{1,3}';
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что метка таблицы сброшена
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять метки таблиц друг у друга
-- Пользователь u_0_00 пытается установить метку на таблицу пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 SET MAC TO '{0,0}';
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что метка таблицы сброшена
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор возвращает метки таблиц test_u_0_00 и test_u_1_11
\connect - postgres
ALTER TABLE test_u_0_00 SET MAC TO '{0,0}';
ALTER TABLE test_u_1_11 SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем таблицу от его имени
CREATE TABLE test_dbsa (data TEXT);
--
-- Проверка метки таблицы test_dbsa
-- Ожидается, что метка таблицы соответствует метке пользователя {2,3}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что он может убрать метку своей таблицы
ALTER TABLE test_dbsa SET MAC TO NULL;
--
-- Проверка метки таблицы test_dbsa
-- Ожидается, что таблица не имеет метки
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что он может сменить метку своей таблицы на {1,1}
ALTER TABLE test_dbsa SET MAC TO '{1,1}';
--
-- Проверка метки таблицы test_dbsa
-- Ожидается, что метка таблица равна {1,1}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Проверяем, что пользователь dbsa с флагом CAP_CHMAC не имеет возможности поменять метки таблиц других пользователей
-- Пользователь dbsa пытается сбросить метку у таблицы пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что метка таблицы осталась той же {0,0}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Пользователь dbsa пытается сбросить метку у таблицы пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что метка таблицы осталась той же {1,3}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Проверяем, что администратор может менять метки у любых таблиц
\connect - postgres
--
-- Администратор сбрасывает метку у таблицы пользователя u_0_00
ALTER TABLE test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что таблица потеряла метку
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Администратор сбрасывает метку у таблицы пользователя u_1_11
ALTER TABLE test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что таблица потеряла метку
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор сбрасывает метку у таблицы пользователя dbsa
ALTER TABLE test_dbsa SET MAC TO NULL;
--
-- Проверка метки таблицы test_dbsa
-- Ожидается, что таблица потеряла метку
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Администратор устанавливает метку у таблицы пользователя u_0_00 в {1,1}
ALTER TABLE test_u_0_00 SET MAC TO '{1,1}';
--
-- Проверка метки таблицы test_u_0_00
-- Ожидается, что таблица приобрела метку {1,1}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_0_00'::regclass::oid;
--
-- Администратор устанавливает метку у таблицы пользователя u_1_11 в {2,2}
ALTER TABLE test_u_1_11 SET MAC TO '{2,2}';
--
-- Проверка метки таблицы test_u_1_11
-- Ожидается, что таблица приобрела метку {2,2}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_u_1_11'::regclass::oid;
--
-- Администратор устанавливает метку у таблицы пользователя dbsa в {3,3}
ALTER TABLE test_dbsa SET MAC TO '{3,3}';
--
-- Проверка метки таблицы test_dbsa
-- Ожидается, что таблица приобрела метку {3,3}
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_dbsa'::regclass::oid;
--
-- Создаем последовательность
CREATE SEQUENCE test_seq;
--
-- Проверка метки последовательности test_seq
-- Ожидается, что последовательность приобрела метку {0,0} в случае установленного
-- конфигурационного параметра ac_enable_sequence_mac
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_seq'::regclass::oid;

-- Попытка установить метку последовательности
-- Ожидается успех в случае установленного конфигурационного параметра ac_enable_sequence_mac
-- иначе ожидается ошибка доступа
ALTER TABLE test_seq SET MAC TO '{1,0}';
--
-- Проверка метки последовательности test_seq
-- Ожидается, что последовательность приобрела метку {1,0} в случае установленного
-- конфигурационного параметра ac_enable_sequence_mac
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'test_seq'::regclass::oid;
--
-- Попытка установить метку индекса
-- Ожидается ошибка доступа
CREATE INDEX test_idx ON test_u_0_00(data);
ALTER TABLE test_idx SET MAC TO '{0,0}';
--
-- Проверяем, что метки у системных таблиц изменить нельзя
-- Попытки установки метки системным таблицам
-- Ожидается ошибка доступа
ALTER TABLE pg_catalog.pg_class SET MAC TO '{0,0}';
ALTER TABLE information_schema.sql_features SET MAC TO '{0,0}';
--
-- Проверка выполнения команды ALTER TABLE ... SET MAC TO ... для видов
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем от его имени вид
CREATE VIEW view_test_u_0_00 AS SELECT * from pg_user;
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что вид приобрел метку пользователя {0,0}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку вида
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что метка вида не изменилась {0,0}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Cоздаем вид от его имени
CREATE VIEW view_test_u_1_11 AS SELECT * from pg_user;
--
-- Проверка метки вида view_test_u_1_11
-- Ожидается, что вид приобрел метку пользователя {1,3}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, чтобы он так же не мог изменить метку своего вида
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_1_11
-- Ожидается, что метка вида не изменилась {1,3}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, что пользователи не имеют возможности поменять метки видов друг у друга
--
-- Администратор сбрасывает метку видов view_test_u_0_00 и view_test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 SET MAC TO NULL;
ALTER TABLE view_test_u_1_11 SET MAC TO NULL;
--
-- Подключаемся как пользователь u_1_11 с ненулевой меткой {1,3}
\connect - u_1_11
--
-- Пользователь u_1_11 пытается установить метку на вид пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 SET MAC TO '{1,3}';
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что метка вида сброшена
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, что пользователи не имеют возможности поменять метки видов друг у друга
-- Пользователь u_0_00 пытается установить метку на вид пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 SET MAC TO '{0,0}';
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что метка таблицы сброшена
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор возвращает метки видов view_test_u_0_00 и view_test_u_1_11
\connect - postgres
ALTER TABLE view_test_u_0_00 SET MAC TO '{0,0}';
ALTER TABLE view_test_u_1_11 SET MAC TO '{1,3}';
--
-- Подключаемся как пользователь dbsa с флагом CAP_CHMAC и меткой {2,3}
\connect - dbsa
--
-- Создаем вид от его имени
CREATE VIEW view_test_dbsa AS SELECT * from pg_user;
--
-- Проверка метки вида view_test_dbsa
-- Ожидается, что вид приобрел метку пользователя {2,3}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Проверяем, что он может убрать метку своего вида
ALTER TABLE view_test_dbsa SET MAC TO NULL;
--
-- Проверка метки вида view_test_dbsa
-- Ожидается, что вид потерял метку
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Проверяем, что он может сменить метку на {1,1}
ALTER TABLE view_test_dbsa SET MAC TO '{1,1}';
--
-- Проверка метки вида view_test_dbsa
-- Ожидается, что вид пориобрел метку {1,1}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Но не может менять метки видов других пользователей
-- Пользователь dbsa пытается сбросить метку у вида пользователя u_0_00
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что метка вида не изменилась {0,0}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Пользователь dbsa пытается сбросить метку у вида пользователя u_1_11
-- Ожидается ошибка доступа
ALTER TABLE view_test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_1_11
-- Ожидается, что метка вида не изменилась {1,3}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Проверяем, что администратор может менять метки у любых видов
\connect - postgres
--
-- Администратор сбрасывает метку у вида пользователя u_0_00
ALTER TABLE view_test_u_0_00 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что вид потерял метку
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Администратор сбрасывает метку у вида пользователя u_1_11
ALTER TABLE view_test_u_1_11 SET MAC TO NULL;
--
-- Проверка метки вида view_test_u_1_11
-- Ожидается, что вид потерял метку
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор сбрасывает метку у вида пользователя dbsa
ALTER TABLE view_test_dbsa SET MAC TO NULL;
--
-- Проверка метки вида view_test_dbsa
-- Ожидается, что вид потерял метку
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
-- Администратор устанавливает метку вида пользователя u_0_00 в {1,1}
ALTER TABLE view_test_u_0_00 SET MAC TO '{1,1}';
--
-- Проверка метки вида view_test_u_0_00
-- Ожидается, что вид приобрел метку {1,1}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_0_00'::regclass::oid;
--
-- Администратор устанавливает метку вида пользователя u_1_11 в {2,2}
ALTER TABLE view_test_u_1_11 SET MAC TO '{2,2}';
--
-- Проверка метки вида view_test_u_1_11
-- Ожидается, что вид приобрел метку {2,2}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_u_1_11'::regclass::oid;
--
-- Администратор устанавливает метку вида пользователя dbsa в {3,3}
ALTER TABLE view_test_dbsa SET MAC TO '{3,3}';
--
-- Проверка метки вида view_test_dbsa
-- Ожидается, что вид приобрел метку {3,3}
SELECT relname AS "Наименование вида", relmaclabel AS "Метка" FROM pg_class WHERE oid = 'view_test_dbsa'::regclass::oid;
--
\i 'support/done.sql'
