-- RBT-TEST - МПРД при доступе к столбцам
-- Проверка работы МПРД при доступе к столбцам
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Проверка работы МПРД для столбцов при наличии дискреционных прав на таблицу
--
-- Подключается администратор postgres
\connect - postgres
--
-- Устанавливает метку столбца "идентификатор" в {1,1}
ALTER TABLE "Проверка" ALTER COLUMN "идентификатор" SET MAC TO '{1, 1}';
--
-- Проверка метки столбца "идентификатор"
-- Ожидается, что метка столбца установлена в {1,1}
SELECT relname AS "Наименование объекта", attname AS "Наименование столбца", attmaclabel AS "Метка"
FROM pg_attribute JOIN pg_class ON oid = attrelid
WHERE attrelid = 'Проверка'::regclass::oid and attnum >= 0;
--
-- Устанавливает метку таблицы в {1,1}
ALTER TABLE "Проверка" SET MAC TO '{1, 1}';
--
-- Запрещаем доступ к таблице
REVOKE ALL ON TABLE "Проверка" FROM PUBLIC;
--
-- Явно разрешаем доступ к столбцам 
GRANT ALL ("идентификатор", "данные") ON TABLE "Проверка" TO PUBLIC;
--
-- Выполняем попытку чтения от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку чтения от имени пользователя u_0_00 с меткой {0,0}
-- столбца "данные" с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
SELECT "данные" FROM "Проверка" ORDER BY "идентификатор";
--
-- Включаем режим использования меток столбцов
\connect - postgres
ALTER TABLE "Проверка" ENABLE COLUMN MACS;
--
-- Разрешаем доступ к таблице 
REVOKE ALL ("идентификатор", "данные") ON TABLE "Проверка" FROM PUBLIC;
GRANT ALL ON TABLE "Проверка" TO PUBLIC;
--
-- Выполняем попытку чтения от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку чтения от имени пользователя u_0_00 с меткой {0,0) только столбца с меткой {0,0}
-- Ожидается 2 записи
SELECT maclabel, "данные" FROM "Проверка";
--
-- Выполняем попытку чтения от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9 записей
\connect - u_1_11
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку чтения от имени пользователь dbsa с флагами игнорирования МПРД
-- Ожидается 10 записей
\connect - dbsa
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку вставки от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Проверка" VALUES (11, USER);
--
-- Проверяем что запись от имени пользователя u_0_00 с меткой {0,0} была вставлена
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку вставки от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается ошибка доступа, так как метка столбца "данные" равна {0,0}
\connect - u_1_01
INSERT INTO "Проверка" VALUES (12, USER);
--
-- Выполняем попытку вставки от имени пользователя u_1_01 с меткой {1,1}
-- в столбец с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
INSERT INTO "Проверка" ("идентификатор") VALUES (12);
--
-- Проверяем что запись от имени пользователя u_1_01 с меткой {1,1} была вставлена
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку вставки от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
INSERT INTO "Проверка" VALUES (13, USER);
--
-- Выполняем попытку вставки от имени пользователя u_1_11 с меткой {1,3}
-- в столбец с меткой {1,1}
-- Ожидается ошибка доступа
\connect - u_1_11
INSERT INTO "Проверка" ("идентификатор") VALUES (13);
--
-- Выполняем попытку вставки от имени пользователя dbsa с флагами игнорирования МПРД
-- Ожидается успех
\connect - dbsa
INSERT INTO "Проверка" VALUES (14, USER);
--
-- Проверяем содержимое таблицы администратором postgres
-- Ожидаются строки, вставленные пользователями с метками меньшими или равными {1,1}
-- при вставке в столбцы с соответстсвующими меткаим
-- и пользователем dbsa с флагами игнорирования МПРД
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку модификации от имени пользователя u_0_00 с меткой {0,0}
-- столбца с меткой {1,1}
-- Ожидается ошибка доступа
\connect - u_0_00
UPDATE "Проверка" SET "идентификатор" = "идентификатор"+100;
--
-- Выполняем попытку модификации от имени пользователя u_0_00 с меткой {0,0}
-- столбца с меткой {0,0}
-- Ожидается успех
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем что модификация от имени пользователя u_0_00 с меткой {0,0} была выполнена
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку модификации от имени пользователя u_1_01 с меткой {1,1}
-- столбца с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
UPDATE "Проверка" SET "идентификатор" = "идентификатор"+100;
--
-- Выполняем попытку модификации от имени пользователя u_1_01 с меткой {1,1}
-- столбца с меткой {0,0}
-- Ожидается ошибка доступа
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем что модификация от имени пользователя u_1_01 с меткой {1,1} была выполнена
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку модификации от имени пользователя u_1_11 с меткой {1,3}
-- столбца с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Выполняем попытку модификации от имени пользователя u_1_11 с меткой {1,3}
-- столбца с меткой {1,1}
-- Ожидается ошибка доступа
\connect - u_1_11
UPDATE "Проверка" SET "идентификатор" = "идентификатор"+100;
--
-- Проверяем что модификация от имени пользователя u_1_11 с меткой {1,3} не была выполнена
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку модификации от имени пользователя dbsa с флагами игнорирования МПРД
-- Ожидается успех
\connect - dbsa
UPDATE "Проверка" SET "идентификатор" = "идентификатор"+100, "данные" = "данные" || '+' || USER;
--
-- Проверяем что модификация от имени пользователя dbsa с флагами игнорирования МПРД была выполнена для обоих столбцов
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";
--
-- Выполняем попытку удаления от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
DELETE FROM "Проверка";
--
-- Выполняем попытку удаления от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается ошибка доступа
\connect - u_1_01
DELETE FROM "Проверка";
--
-- Выполняем попытку удаления от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
DELETE FROM "Проверка";
--
-- Выполняем попытку удаления от имени пользователя dbsa с флагами игнорирования МПРД
-- Ожидается успех
\connect - dbsa
DELETE FROM "Проверка";
--
-- Проверяем что данные были удалены
\connect - postgres
SELECT maclabel, * FROM "Проверка" ORDER BY "идентификатор";

--
\i 'support/done.sql'
