-- RBT-TEST - Запрет команды COPY TO <файл> WITH MACS
-- Проверка работы МПРД при запрете использования команды COPY TO <файл> WITH MACS
-- при сброшенном параметре разрешения работы команды COPY с файлами ac_enable_copy_to_file
-- Перед выполнением теста папка /tmp/ должна быть очищена от файлов test*.data, создаваемых тестом
-- Для возможности выполнения теста, с помощью visudo для тестирующего пользователя должна быть добавлена строчка "{testuser}	ALL=NOPASSWD: /bin/rm
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/disable-copy-to-file.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Удаляем временные файлы
\! sudo rm -f /tmp/test*.data
--
-- Отбираем права у PUBLIC на тестовую таблицу
\connect - postgres
REVOKE ALL ON "Проверка" FROM PUBLIC;
--
-- Подключение пользователя u_0_00 c меткой {0,0}
\connect - u_0_00
--
-- Попытка выполнить команду COPY в файл не администратором
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp/test.data';
--
-- Подключается суперюзер postgres 
\connect - postgres
--
-- Попытка выполнить команду COPY в файл администратором
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp/test.data';
--
-- Просматриваем результирующий файл
-- Ожидается, что файл не создан
\! cat "/tmp/test.data"
--
-- Попытка выполнить команду COPY с выводом меток в файл
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp/test_wm.data' WITH MACS;
--
-- Просматриваем результирующий файл
-- Ожидается, что файл не создан
\! cat "/tmp/test_wm.data"
--
-- Попытка выполнить команду COPY в бинарном формате
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp/test_b.data' BINARY;
--
-- Просматриваем результирующий файл
-- Ожидается, что файл не создан
\! echo -e | cat "/tmp/test_b.data" - 
--
-- Попытка выполнить команду COPY с выводом меток в файл в бинарном формате 
-- Ожидается ошибка доступа
COPY "Проверка" TO '/tmp/test_b_wm.data' WITH MACS BINARY;
--
-- Просматриваем результирующий файл
-- Ожидается, что файл не создан
\! echo -e | cat "/tmp/test_b_wm.data" -
--
-- Очищаем таблицу
DELETE FROM "Проверка";
--
-- Подключение пользователя u_2_11 c меткой {2,3}
\connect - u_2_11
-- 
-- Пытаемся заполнить таблицу из файла без меток
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test.data';
--
-- Пытаемся заполнить таблицу из файла с метками без меток
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test_wm.data';
--
-- Пытаемся заполнить таблицу из файла с метками
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test_wm.data' WITH MACS;
--
-- Проверяем содержимое таблицы
-- Ожидается, что данные не вставились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключается суперюзер postgres 
\connect - postgres
--
-- Пытаемся заполнить таблицу из бинарного файла без меток
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test_b.data' BINARY;
--
-- Пытаемся заполнить таблицу из бинарного файла с метками без меток
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test_b_wm.data' BINARY;
--
-- Пытаемся заполнить таблицу из бинарного файла с метками
-- Ожидается ошибка доступа
COPY "Проверка" FROM '/tmp/test_b_wm.data' WITH MACS BINARY;
--
-- Проверяем содержимое таблицы
-- Ожидается, что данные не вставились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Удаляем временные файлы
\! sudo rm -f /tmp/test*.data
--
\i 'support/done.sql'
