-- RBT-TEST - МПРД при создании объектов
-- Проверка работы МПРД в разных режимах создания объектов
-- при установленном параметре разрешения использования меток последовательностей ac_enable_sequence_mac
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
\i 'support/enable-sequence-mac.sql'
--
-- Подключаемся как пользователь c меткой {1,1}
\connect - u_1_01
--
-- Создаем от его имени таблицу с параметрами по умолчанию
CREATE TABLE test_u_1_01_1 (data TEXT);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_1'::regclass::oid;
--
-- Проверяем режим создания таблицы, записи которой защищены мандатной меткой
CREATE TABLE test_u_1_01_2 (data TEXT) WITH (MACS=TRUE);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_2'::regclass::oid;
--
-- Проверяем режим создания таблицы с явным заданием отсутствия мандатных меток у записей
CREATE TABLE test_u_1_01_3 (data TEXT) WITH (MACS=FALSE);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_3'::regclass::oid;
--
-- Проверяем режим создания вида
CREATE VIEW test_u_1_01_4 AS SELECT * FROM test_u_1_01_1;
--
-- Ожидается, что метка созданного вида соответствует метке пользователя {1,1}
-- и записи вида незащищены (меток не имеют)
SELECT relname AS "Наименование вида", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_4'::regclass::oid;
--
-- Проверяем режим создания последовательности
CREATE SEQUENCE test_u_1_01_5;
--
-- В случае установки конфигурационного параметра разрешения использования меток последовательностей
-- ac_enable_sequence_mac ожидается, что метка созданной последовательности соответствует метке пользователя {1,1}
-- иначе последовательности не защищаются (метки не имеют)
--
-- записи последовательности незащищены (меток не имеют)
SELECT relname AS "Наименование последовательности", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_5'::regclass::oid;
--
-- Проверяем режим создания индекса
CREATE INDEX test_u_1_01_6 ON test_u_1_01_1 (data);
--
-- Проверяем, что индексы не защищаются (метки не имеют)
-- и записи индекса незащищены (меток не имеют)
SELECT relname AS "Наименование индекса", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_01_6'::regclass::oid;
--
-- Даем все права на таблицу test_u_1_01_1 и test_u_1_01_2
GRANT ALL ON "test_u_1_01_1" TO PUBLIC;
GRANT ALL ON "test_u_1_01_2" TO PUBLIC;
--
-- Подключаемся как пользователь c меткой {0,0}
\connect - u_0_00
--
-- Проверяем режим создания таблицы на основании существующей
-- Ожидается ошибка доступа
CREATE TABLE test_u_0_00_1 (LIKE test_u_1_01_1);
--
-- Подключаемся как пользователь c меткой {1,3}
\connect - u_1_11
--
-- Проверяем режим создания таблицы на основании существующей
-- Ожидается успех
CREATE TABLE test_u_1_11_1 (LIKE test_u_1_01_1);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,3}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_11_1'::regclass::oid;
--
-- Проверяем режим создания таблицы, записи которой защищены мандатной меткой, на основе таблицы без защищеных записей
CREATE TABLE test_u_1_11_2 (LIKE test_u_1_01_1) WITH (MACS=TRUE);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,3}
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_11_2'::regclass::oid;
--
-- Проверяем режим создания таблицы на основе таблицы, записи которой защищены мандатной меткой
CREATE TABLE test_u_1_11_3 (LIKE test_u_1_01_2);
--
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,3}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'test_u_1_11_3'::regclass::oid;
--
--
\i 'support/done.sql'
