-- RBT-TEST - CREATE TABLE ? AS ?
-- Проверка сохранения атрибутов безопасности при выполнении команды CREATE TABLE ... AS ... и SELECT ... INTO ...
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Подключаемся пользователем u_1_10 с меткой {1,2}
\connect - u_1_10
--
-- Создаем таблицу на основе существующей с помощью CREATE TABLE ... AS ...
CREATE TABLE "Проверка2" AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка2'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, в таблицу попали только доступные по МПРД
SELECT * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка3" WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка3'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали только доступные по МПРД записи, с метками пользователя {1,2}
SELECT maclabel, * FROM "Проверка3" ORDER BY "идентификатор";
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка4" WITH (MACS=TRUE) AS SELECT * FROM "Проверка2";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка4'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали только доступные по МПРД записи, с метками пользователя {1,2}
SELECT maclabel, * FROM "Проверка4" ORDER BY "идентификатор";
--
-- Подключаемся пользователем dbsa с привилегией CHMAC
\connect - dbsa
--
-- Создаем таблицу с защищенными строками на основе существующей
CREATE TABLE "Проверка5" WITH (MACS=TRUE) AS SELECT * FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя dbsa
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка5'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается, что в таблицу попали записи c исходными метками
SELECT maclabel, * FROM "Проверка5" ORDER BY "идентификатор";
--
-- Подключаемся пользователем u_1_10 с меткой {1,2}
\connect - u_1_10
--
-- Создаем таблицу на основе существующей с помощью SELECT ... INTO ...
SELECT *  INTO "Проверка6" FROM "Проверка";
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,2}
-- и записи таблицы незащищены (меток не имеют)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка6'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, в таблицу попали только доступные по МПРД
SELECT * FROM "Проверка6" ORDER BY "идентификатор";
--
\i 'support/done.sql'
