-- RBT-TEST - МПРД при вставке (INSERT)
-- Проверка работы МПРД при добавлении данных командой INSERT
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
--
-- Администратор postgres устанавливает метку {1, 1} на таблицу
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{1,1}';
--
-- Пытаемся вставить в таблицу данные от имени разных пользователей
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается успех
\connect - u_0_00
INSERT INTO "Проверка" VALUES (1, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается успех
\connect - u_0_01
INSERT INTO "Проверка" VALUES (2, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
INSERT INTO "Проверка" VALUES (3, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается ошибка доступа
\connect - u_0_10
INSERT INTO "Проверка" VALUES (4, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается ошибка доступа
\connect - u_1_10
INSERT INTO "Проверка" VALUES (5, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
INSERT INTO "Проверка" VALUES (6, USER || '-insert');
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается ошибка доступа
\connect - u_2_11
INSERT INTO "Проверка" VALUES (7, USER || '-insert');
--
-- Выполняем запрос от имени пользователя dbsa
-- Ожидается успех
\connect - dbsa
SET ac_session_maclabel = '{0, 0}';
INSERT INTO "Проверка" VALUES (8, USER || '-insert-0_00');
SET ac_session_maclabel = '{1, 0}';
INSERT INTO "Проверка" VALUES (9, USER || '-insert-1_00');
SET ac_session_maclabel = '{1, 3}';
INSERT INTO "Проверка" VALUES (10, USER || '-insert-1_11');
--
-- Проверяем выборку данных (правильность заполнения тестовой таблицы)
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 6 записей
\connect - postgres
SELECT * FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем возможность вставки метки пользователем с привилегией назначения метки CHMAC
-- Ожидается успех
\connect - dbsa
SET ac_session_maclabel = '{0, 0}';
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES ('{4,3}', 11, USER || '-insert-4_11');
--
-- Проверяем, что строка была вставлена
-- Ожидается 7 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем невозможность вставки метки пользователем без привилегии назначения метки CHMAC
-- Ожидается ошибка доступа
\connect - u_1_01
INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES ('{1,1}', 12, USER || '-insert-0_00');
--
-- Проверяем, что строка не была вставлена
-- Ожидается 7 записей
\connect - dbsa
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключаемся пользователем u_1_10 с меткой {1,2}
\connect - u_1_01
--
-- Создаем таблицу на основе существующей с помощью CREATE ... (LIKE ...)
CREATE TABLE "Проверка2" (LIKE "Проверка")  WITH(MACS=TRUE);
--
-- Проверяем метку созданной таблицы
-- Ожидается, что метка созданной таблицы соответствует метке пользователя {1,1}
-- и записи таблицы защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relmaclabel AS "Метка", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'Проверка2'::regclass::oid;
--
-- Проверяем содержимое таблицы
-- Ожидается что таблица пуста
SELECT * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
INSERT INTO "Проверка2" SELECT "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка2";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
-- с помощью DISTINCT
INSERT INTO "Проверка2" SELECT DISTINCT "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
-- Очищаем таблицу
DELETE FROM "Проверка2";
--
-- Выполняем вставку данных в таблицу выборкой из другой таблицы
-- с помощью DISTINCT ON
INSERT INTO "Проверка2" SELECT DISTINCT ON ("идентификатор") "идентификатор", "данные" FROM "Проверка";
--
-- Проверяем содержимое таблицы
-- Ожидается 5 записей, доступные по МПРД, с метками пользователя {1,1}
SELECT maclabel, * FROM "Проверка2" ORDER BY "идентификатор";
--
\i 'support/done.sql'
