-- RBT-TEST - МПРД при работе с большими объектами
-- Проверка МПРД при работе с большими объектами
--
\i 'support/init.sql'
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Проверяем, что большие объекты имеют атрибут мандатной метки
-- Ожидается что записи таблиц pg_largeobject и pg_largeobject_metadata защищены (имеют метки)
SELECT relname AS "Наименование таблицы", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE oid = 'pg_largeobject'::regclass::oid or oid = 'pg_largeobject_metadata'::regclass::oid;
-- 
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB" (loid oid, fd integer);
--
-- Отключаем ДПРД и МПРД к созданной таблице
GRANT ALL ON "Проверка BLOB" TO PUBLIC;
--
-- Подключаемся как администратор кластера
\connect - postgres
ALTER TABLE "Проверка BLOB" SET MAC TO NULL;
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Вставляем запись в таблицу pg_largeobject с меткой {0,0} для объекта 200000
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200000, 0, 'test');
SELECT maclabel, * FROM pg_largeobject;
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Создаем объект с идентификатором 200000 из таблицы pg_largeobject
-- ожидается ошибка доступа
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200000);
--
-- Создаем объект с идентификаторм 200001
-- Ожидается успех
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200001);
SELECT * FROM "Проверка BLOB";
--
-- Делаем доступным всем
GRANT ALL ON LARGE OBJECT 200001 to PUBLIC;
--
-- Начинаем транзакцию
BEGIN;
--
-- Открываем объект на чтение и запись
--   INV_READ  = 0x40000
--   INV_WRITE = 0x20000
UPDATE "Проверка BLOB" SET fd = lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Проверка записи, ожидается успех
SELECT lowrite(fd, '
Whose woods these are I think I know,
His house is in the village though.
He will not see me stopping here,
To watch his woods fill up with snow.

My little horse must think it queer,
To stop without a farmhouse near,
Between the woods and frozen lake,
The darkest evening of the year.

He gives his harness bells a shake,
To ask if there is some mistake.
The only other sound''s the sweep,
Of easy wind and downy flake.

The woods are lovely, dark and deep,
But I have promises to keep,
And miles to go before I sleep,
And miles to go before I sleep.

         -- Robert Frost
') FROM "Проверка BLOB";
--
-- Закрываем объект
-- Ожидается 0 - отсутствие ошибок, отрицательное значение - ошибка
SELECT lo_close(fd) FROM "Проверка BLOB";
--
-- Завершаем транзакцию
END;
--
-- Проверяем прямой доступ к данным объектов
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
-- Проверяем чтение/запись для владельца
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Записываем символ
SELECT lowrite(fd, 'n') FROM "Проверка BLOB";
--
-- Читаем значение позиции
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -156, 2) FROM "Проверка BLOB";
--
-- Читаем 35 символов
-- Ожидается изменение буквы в слове на записанное значение
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Проверяем изменение размера объекта
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Увеличиваем
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Уменьшаем
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Проверяем, что пользователь без CAP_CHMAC не имеет возможности менять метку объекта
-- Ожидается ошибка доступа
ALTER LARGE OBJECT 200001 SET MAC TO '{0,0}';
--
-- Подключаемся как администратор кластера postgres
\connect - postgres
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Вставляем запись в таблицу pg_largeobject для объекта 200001
-- Ожидается успех
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, 'test {0,0}');
--
-- Ожидается, что вставленная запись приобретет метку объекта
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем возможность работы с объектом пользователями с другими метками
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что пользователь с меньшей меткой, не может увидеть объект 200001
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся открыть объект на чтение
-- Ожидается ошибка, объект для пользователя недоступен
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'40000' AS integer));
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка, объект для пользователя опять же недоступен
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка, объект для пользователя недоступен
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект виден для пользователя с большей меткой
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся добавить страницу к объекту
-- ожидается ошибка доступа
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, convert_to(USER || ' data', 'UTF8'));
--
-- Пытаемся открыть объект на чтение
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка доступа
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Подключаемся как пользователь dbsa с атрибутами игнорирования МПРД
\connect - dbsa
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Устанавливаем метку сессии в {0,0}
SET ac_session_maclabel = '{0, 0}';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект доступен
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем чтение/запись
BEGIN;
UPDATE "Проверка BLOB" SET fd=lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB";
--
-- Записываем символ
SELECT lowrite(fd, 'v') FROM "Проверка BLOB";
--
-- Читаем значение позиции
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB";
--
-- Читаем 35 символов
-- Ожидается изменение буквы в слове на записанное значение
SELECT loread(fd, 35) FROM "Проверка BLOB";
--
-- Проверяем изменение размера объекта
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Увеличиваем
SELECT lo_truncate(fd, 590) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Уменьшаем
SELECT lo_truncate(fd, 578) FROM "Проверка BLOB";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB";
SELECT lo_tell(fd) FROM "Проверка BLOB";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB";
END;
--
-- Проверяем возможность выгрузки объекта на диск
-- Ожидается ошибка - функция не существует
SELECT lo_export(loid, '/tmp/lotest.txt') FROM "Проверка BLOB";
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB";
--
-- Проверяем возможность загрузки объекта с диска
-- Ожидается ошибка - функция не существует
INSERT INTO "Проверка BLOB" (loid) SELECT lo_import('/tmp/lotest.txt');
--
-- Создаем объект с идентификаторм 200002
-- Ожидается успех
SELECT lo_create(200002);
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект 200002 приобрел метку '{0,0}'
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
-- Проверяем, что пользователь c CAP_CHMAC имеет возможности менять метку своего объекта
-- Ожидается успех
ALTER LARGE OBJECT 200002 SET MAC TO '{2,0}';
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект 200002 приобрел метку '{2,0}'
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
\i 'support/done.sql'

