-- RBT-TEST - МПРД в процедурах на Untrusted PL/Perl
-- Проверка работы МПРД в разных режимах создания хранимых процедур на примере языка plperlu в части прав выполнения
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/init-plperlu.sql'
\i 'support/select-test-table.sql'
--
\connect - postgres
--
-- Администратор postgres создает процедуру count_as_invoker
CREATE FUNCTION count_as_invoker () RETURNS INTEGER AS $$
	my $q = spi_exec_query('SELECT count (*) AS cnt FROM "Проверка"', 1);
	return $q->{rows}->[0]->{cnt};
$$ LANGUAGE plperlu SECURITY INVOKER;
--
-- Результаты вызова count_as_invoker должны полностью повторять результаты простых select'ов
-- Так как выполняются с меткой вызывающего пользователя
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 10
\connect - postgres
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается 2
\connect - u_0_00
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается 3
\connect - u_0_01
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 5
\connect - u_1_01
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается 3
\connect - u_0_10
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается 5
\connect - u_1_10
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9
\connect - u_1_11
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 10
\connect - u_2_11
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 10
\connect - dbsa
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
-- Ожидается 10
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
-- Ожидается 10
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
-- Ожидается 10
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_invoker ();
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c атрибутами игнорирования МПРД
\connect - postgres
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Пользователь dbsa c атрибутами игнорирования МПРД создает процедуру count_as_definer
\connect - dbsa
CREATE FUNCTION count_as_definer () RETURNS INTEGER AS $$
	my $q = spi_exec_query('SELECT count (*) AS cnt FROM "Проверка"', 1);
	return $q->{rows}->[0]->{cnt};
$$ LANGUAGE plperlu SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c атрибутами игнорирования МПРД
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Вне зависимости от вызывающего пользователя данные выбираются с игнорированием метки
-- Во всех случаях ожидается 10
-- Выполняем запрос от имени суперпользователя postgres
\connect - postgres
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
\connect - u_0_00
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
\connect - u_0_01
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
\connect - u_1_01
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
\connect - u_0_10
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
\connect - u_1_10
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
\connect - u_1_11
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
\connect - u_2_11
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa
\connect - dbsa
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_definer ();
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю u_0_00 с минимальной меткой {0,0}
\connect - postgres
ALTER ROLE u_0_00 WITH SUPERUSER;
--
--
-- Пользователь u_0_00 с минимальной меткой {0,0} создает процедуру count_as_definer_u_0_00
\connect - u_0_00
CREATE FUNCTION count_as_definer_u_0_00 () RETURNS INTEGER AS $$
	my $q = spi_exec_query('SELECT count (*) AS cnt FROM "Проверка"', 1);
	return $q->{rows}->[0]->{cnt};
$$ LANGUAGE plperlu SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя u_0_00 с минимальной меткой {0,0}
\connect - postgres
ALTER ROLE u_0_00 WITH NOSUPERUSER;
--
-- Результаты вызова count_as_definer должны полностью соответствовать меткам пользователей
-- Так как у пользователя u_0_00 отсутствуют атрибуты игнорирования МПРД
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 2
\connect - postgres
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается 2
\connect - u_0_00
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается 3
\connect - u_0_01
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 5
\connect - u_1_01
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается 3
\connect - u_0_10
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается 5
\connect - u_1_10
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9
\connect - u_1_11
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 10
\connect - u_2_11
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 10
\connect - dbsa
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
-- Ожидается 2
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
-- Ожидается 3
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
-- Ожидается 9
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_definer_u_0_00 ();
--
\i 'support/done.sql'
