-- RBT-TEST - МПРД в процедурах на PL/pgSQL
-- Проверка работы МПРД в разных режимах создания хранимых процедур на примере языка plpgsql в части прав выполнения
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
\connect - postgres
--
-- Администратор postgres создает процедуру count_as_invoker
CREATE FUNCTION count_as_invoker () RETURNS INTEGER AS '
DECLARE
	результат INTEGER;
BEGIN
	SELECT count (*) INTO результат FROM "Проверка";
	RETURN результат;
END;
' LANGUAGE plpgsql SECURITY INVOKER;
--
-- Результаты вызова count_as_invoker должны полностью повторять результаты простых select'ов
-- Так как выполняются с меткой вызывающего пользователя
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 10
\connect - postgres
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается 2
\connect - u_0_00
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается 3
\connect - u_0_01
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 5
\connect - u_1_01
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается 3
\connect - u_0_10
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается 5
\connect - u_1_10
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9
\connect - u_1_11
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 10
\connect - u_2_11
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 10
\connect - dbsa
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
-- Ожидается 10
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
-- Ожидается 10
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_invoker ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
-- Ожидается 10
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_invoker ();

--
-- Пользователь dbsa c атрибутами игнорирования МПРД создает процедуру count_as_definer
CREATE FUNCTION count_as_definer () RETURNS INTEGER AS '
DECLARE
	результат INTEGER;
BEGIN
	SELECT count (*) INTO результат FROM "Проверка";
	RETURN результат;
END;
' LANGUAGE plpgsql SECURITY DEFINER;

--
-- Вне зависимости от вызывающего пользователя данные выбираются с игнорированием метки
-- Во всех случаях ожидается 10
-- Выполняем запрос от имени суперпользователя postgres
\connect - postgres
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
\connect - u_0_00
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
\connect - u_0_01
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
\connect - u_1_01
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
\connect - u_0_10
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
\connect - u_1_10
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
\connect - u_1_11
SELECT count_as_definer ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
\connect - u_2_11
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa
\connect - dbsa
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_definer ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_definer ();

--
-- подключаемся как пользователь u_0_00 с минимальной меткой {0,0}
\connect - u_0_00
--
-- Пользователь пользователь u_0_00 с минимальной меткой создает процедуру count_as_definer_u_0_00
CREATE FUNCTION count_as_definer_u_0_00 () RETURNS INTEGER AS '
DECLARE
	результат INTEGER;
BEGIN
	SELECT count (*) INTO результат FROM "Проверка";
	RETURN результат;
END;
' LANGUAGE plpgsql SECURITY DEFINER;

--
-- Результаты вызова count_as_definer должны полностью соответствовать меткам пользователей
-- Так как у пользователя u_0_00 отсутствуют атрибуты игнорирования МПРД
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 2
\connect - postgres
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается 2
\connect - u_0_00
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается 3
\connect - u_0_01
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 5
\connect - u_1_01
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается 3
\connect - u_0_10
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается 5
\connect - u_1_10
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9
\connect - u_1_11
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 10
\connect - u_2_11
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 10
\connect - dbsa
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {0,0}
-- Ожидается 2
SET ac_session_maclabel = '{0, 0}';
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,0}
-- Ожидается 3
SET ac_session_maclabel = '{1, 0}';
SELECT count_as_definer_u_0_00 ();
--
-- Выполняем запрос от имени администратора dbsa с меткой {1,3}
-- Ожидается 9
SET ac_session_maclabel = '{1, 3}';
SELECT count_as_definer_u_0_00 ();

--
\i 'support/done.sql'
