-- RBT-TEST - МПРД в триггерах на PL/Perl
-- Проверка работы МПРД в триггерах на языке PL/Perl
--
\i 'support/init.sql'
\i 'support/init-plperl.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Подключается пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1';
	return "MODIFY";
END;$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФунк1" () TO PUBLIC;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФунк1"();
--
-- Проверяем модификацию данных
-- Пользователи меняют данные в соответствии с МПРД
-- Ожидается, что метки записей не изменятся
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что метки строк не изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Пользователь dbsa c максимальной меткой создает процедуру count_as_definer с атрибутами создающего пользователя
\connect - dbsa
CREATE FUNCTION count_as_definer () RETURNS INTEGER AS $$
	my $q = spi_exec_query('SELECT count (*) AS cnt FROM "Проверка"', 1);
	return $q->{rows}->[0]->{cnt};
$$ LANGUAGE plperl SECURITY DEFINER;
--
-- Меняем определение триггерной функции с атрибутами создающего пользователя
\connect - u_0_00
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1+' . $_AC->{user_name};
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка" VALUES ($1, USER || \'-ТрФунк1\')', 'INT4');
	my $q = spi_exec_prepared( $x, $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступна для обновления одна запись с меткой {1,2}, соответственно появляется 1 новая запись с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 13 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 26 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
-- и имя создававшего функцию пользователя
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Делаем владельцем функции пользователя с атрибутами игнорирования МПРД
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO dbsa;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1+' . $_AC->{user_name};
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка" VALUES ($1, USER || \'-ТрФунк1\')', 'INT4');
	my $q = spi_exec_prepared( $x, $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступна для обновления одна запись с меткой {1,2}, соответственно появляется 1 новая запись с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 13 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 26 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Делаем владельцем функции пользователя с меткой {1,2}
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO u_1_10;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - u_1_10
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1+' . $_AC->{user_name};
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка" VALUES ($1, USER || \'-ТрФунк1\')', 'INT4');
	my $q = spi_exec_prepared( $x, $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
-- Ожидается ошибка доступа, так как применяются атрибуты пользователя создавшего функцию, а не dbsa
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Делаем владельцем функции пользователя с атрибутами игнорирования МПРД
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO dbsa;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1+' . $_AC->{user_name};
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка" VALUES ($1, USER || \'-ТрФунк1\')', 'INT4');
	my $q = spi_exec_prepared( $x, $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- Ожидается 12 записей
-- и именем создававшего функцию пользователя
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 24 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Меняем определение триггерной функции для проверки невозможности изменения метки обрабатываемой записи
-- даже если она создается с атрибутами игнорирования МПРД
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{maclabel} = '{1, 0}';
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES ($1, $2, USER || \'-ТрФунк1\')', 'maclabel', 'INT4');
	my $q = spi_exec_prepared( $x, $_TD->{old}{maclabel}, $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
-- Ожидается ошибка доступа, так как пользователь не обладает привилегией явно задавать метку
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что данные не изменились
-- Ожидается 10 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что данные не изменились
-- Ожидается 10 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 20 записей
-- При этом записи меняют метки на указанную в триггере
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Меняем определение триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE plperl AS $$
	$_TD->{new}{"данные"} .= '-ТрФунк1+' . $_AC->{user_name} . '-' . $_TD->{old}{maclabel};
	my $newid = spi_exec_query('SELECT count_as_definer ()+1 AS newid FROM "Проверка"', 1);
	my $x = spi_prepare('INSERT INTO "Проверка"(maclabel, "идентификатор", "данные") VALUES ($1, $2, USER || \'-ТрФунк1\')', 'maclabel', 'INT4');
	my $q = spi_exec_prepared( $x, '{1, 0}', $newid->{rows}->[0]->{newid});
	return "MODIFY";
END;$$ SECURITY DEFINER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что 2 строки было обновлено и 2 добавлены с меткой, указанной в триггере
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи, новые записи приобрели метку, указанную в триггере
-- Ожидается 24 записи
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
\i 'support/done.sql'
