-- RBT-TEST - МПРД в триггерах на Untrusted PL/Tcl
-- Проверка работы МПРД в триггерах на языке Untrusted PL/Tcl
--
\i 'support/init.sql'
\i 'support/init-pltclu.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю u_0_00 с минимальной меткой {0,0}
\connect - postgres
ALTER ROLE u_0_00 WITH SUPERUSER;
--
-- Подключается пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем триггерную функцию, выполняющуюся с атрибутами вызывающего пользователя
CREATE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	set NEW(данные) [join [list $NEW(данные) ТрФунк1] -]
	return [array get NEW]
$$ SECURITY INVOKER;
GRANT EXECUTE ON FUNCTION "ТрФунк1" () TO PUBLIC;
--
-- Отбираем права администратора у пользользователя u_0_00 с минимальной меткой {0,0}
\connect - postgres
ALTER ROLE u_0_00 WITH NOSUPERUSER;
--
-- Создаем триггер
CREATE TRIGGER "ТриггерBU"
  BEFORE UPDATE
  ON "Проверка"
  FOR EACH ROW
  EXECUTE PROCEDURE "ТрФунк1"();
--
-- Проверяем модификацию данных
-- Пользователи меняют данные в соответствии с МПРД
-- Ожидается, что метки записей не изменятся
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что метки строк не изменились
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Пользователь dbsa c максимальной меткой создает процедуру count_as_definer с атрибутами создающего пользователя
\connect - dbsa
CREATE FUNCTION count_as_definer() RETURNS integer AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD(plan)]} {
	    set GD(plan) [ spi_prepare \
		"SELECT count(*) AS cnt FROM Проверка" \
	    {}]
	}

	spi_execp -count 1 $GD(plan)
	return $cnt
$$ LANGUAGE pltclu SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю u_0_00 с минимальной меткой {0,0}
ALTER ROLE u_0_00 WITH SUPERUSER;
--
-- Меняем определение триггерной функции с атрибутами создающего пользователя
\connect - u_0_00
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка VALUES (\$1,\$2)" \
	    {int4 bpchar}]
	}

	set NEW(данные) [join [list [join [list $NEW(данные) ТрФунк1] -] $AC_USER_NAME] +]

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list $newid [join [list $AC_USER_NAME ТрФунк1] -]]

	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя u_0_00 с минимальной меткой {0,0}
\connect - postgres
ALTER ROLE u_0_00 WITH NOSUPERUSER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступна для обновления одна запись с меткой {1,2}, соответственно появляется 1 новая запись с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 13 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 26 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
-- и имя создававшего функцию пользователя
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Делаем владельцем функции пользователя с атрибутами игнорирования МПРД
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO dbsa;
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка VALUES (\$1,\$2)" \
	    {int4 bpchar}]
	}

	set NEW(данные) [join [list [join [list $NEW(данные) ТрФунк1] -] $AC_USER_NAME] +]

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list $newid [join [list $AC_USER_NAME ТрФунк1] -]]

	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступна для обновления одна запись с меткой {1,2}, соответственно появляется 1 новая запись с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 13 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 26 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Делаем владельцем функции пользователя с меткой {1,2}
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO u_1_10;
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю u_1_10 с меткой {1,2}
ALTER ROLE u_1_10 WITH SUPERUSER;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - u_1_10
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка VALUES (\$1,\$2)" \
	    {int4 bpchar}]
	}

	set NEW(данные) [join [list [join [list $NEW(данные) ТрФунк1] -] $AC_USER_NAME] +]

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list $newid [join [list $AC_USER_NAME ТрФунк1] -]]

	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя u_1_10 с меткой {1,2} 
\connect - postgres
ALTER ROLE u_1_10 WITH NOSUPERUSER;
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- и именем создававшего функцию пользователя
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
-- Ожидается ошибка доступа, так как применяются атрибуты пользователя создавшего функцию, а не dbsa
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Делаем владельцем функции пользователя с атрибутами игнорирования МПРД
\connect - postgres
ALTER FUNCTION "ТрФунк1"() OWNER TO dbsa;
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c максимальной меткой
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Меняем определение из под него триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка VALUES (\$1,\$2)" \
	    {int4 bpchar}]
	}

	set NEW(данные) [join [list [join [list $NEW(данные) ТрФунк1] -] $AC_USER_NAME] +]

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list $newid [join [list $AC_USER_NAME ТрФунк1] -]]

	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновление осуществляется в соответствии с МПРД, при этом добавляется соответствующее количество записей
-- Доступно для обновления две записи с меткой {0,0}, соответственно появляется 2 новых записи с меткой пользователя
-- Ожидается 12 записей
-- и именем создававшего функцию пользователя
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 24 записей
-- При этом записи предыдущих пользователей сохраняют свои метки, а вновь добавляемые приобретают максимальную метку пользователя dbsa
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Меняем определение триггерной функции для проверки невозможности изменения метки обрабатываемой записи
-- даже если она создается с атрибутами игнорирования МПРД
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка (maclabel, идентификатор, данные) VALUES (\$1,\$2,\$3)" \
	    {maclabel int4 bpchar}]
	}

	set NEW(maclabel) "{1,0}"

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list $OLD(maclabel) $newid [join [list $AC_USER_NAME ТрФунк1] -]]
	
	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
-- Ожидается ошибка доступа, так как пользователь не обладает привилегией явно задавать метку
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что данные не изменились
-- Ожидается 10 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что данные не изменились
-- Ожидается 10 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи
-- Ожидается 20 записей
-- При этом записи меняют метки на указанную в триггере
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Очищаем таблицу
\connect - postgres
DELETE FROM "Проверка";
--
-- Отключаем контроль МПРД к таблице
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
\i 'support/init-test-table.sql'
--
-- Проверяем, что метки строк соответствуют меткам пользователей
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Устанавливаем на таблицу метку {0,0}
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{0,0}';
--
-- Для возможности создания процедуры на недоверенном языке временно предоставляются
-- права администратора пользользователю dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH SUPERUSER;
--
-- Меняем определение триггерной функции с атрибутами создающего пользователя
\connect - dbsa
CREATE OR REPLACE FUNCTION "ТрФунк1"() RETURNS trigger LANGUAGE pltclu AS $$
	#
	# Подготовка плана при первом вызове
	#
	if {![info exists GD]} {
	    # План получения количества строк в таблице "Проверка"
	    set GD(plancnt) [ spi_prepare \
		"SELECT count_as_definer ()+1 AS newid FROM Проверка" \
	    {}]

	    # План вставки новой строки в таблицу "Проверка"
	    set GD(planins) [ spi_prepare \
		"INSERT INTO Проверка (maclabel, идентификатор, данные) VALUES (\$1,\$2,\$3)" \
	    {maclabel int4 bpchar}]
	}

	set NEW(данные) [join [list [join [list [join [list $NEW(данные) ТрФунк1] -] $AC_USER_NAME] +] $OLD(maclabel)] -]

	spi_execp -count 1 $GD(plancnt)

	spi_execp -count 1 $GD(planins) [list "{1,0}" $newid [join [list $AC_USER_NAME ТрФунк1] -]]
	
	return [array get NEW]
$$ SECURITY DEFINER;
--
-- Отбираем права администратора у пользользователя dbsa c максимальной меткой
\connect - postgres
ALTER ROLE dbsa WITH NOSUPERUSER;
--
-- Проверяем модификацию данных
-- Попытка обновления пользователем u_0_00 с меткой {0,0} 
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что 2 строки было обновлено и 2 добавлены с меткой, указанной в триггере
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем u_1_10 меткой {1,2} 
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Проверяем, что новых данных не появилось
-- Ожидается 12 записей
\connect - postgres
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка обновления пользователем dbsa с атрибутами игнорирования МПРД
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Обновлению подвержены все записи, новые записи приобрели метку, указанную в триггере
-- Ожидается 24 записи
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
\i 'support/done.sql'
