-- RBT-TEST - МПРД при обновлении (UPDATE)
-- Проверка работы МПРД при обновлении данных командой UPDATE
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/select-test-table.sql'
--
-- Проверяем модификацию данных
-- Пользователи меняют данные
-- Ожидается, что метки записей не изменятся
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;

--
-- Проверяем, что обновление данных никак не повлияло на их зашиту
-- Ожидается, что суперюзер postgres и пользователь dbsa с флагами игнорирования МПРД CAP_IGNMACLVL и CAP_IGNMACCAT смогли обновить все записи,
-- остальные пользователи смогли обновить только записи своего уровня, т.е. не смогли осуществить запись выше своего уровня или ниже
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Администратор postgres устанавливает метку {1, 1} на таблицу
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO '{1,1}';
--
-- Проверяем модификацию данных
-- Пользователи меняют данные
-- Ожидается, что изменить данные смогут только пользователи postgres и dbsa с установленными флагами игнорирования МПРД
-- и пользователь с меткой {1,1} для остальных запросов ожидается ошибка доступа
--
-- Попытка обновления пользователем postgres с флагами игнорирования МПРД
-- Ожидается успех
\connect - postgres
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_0_00 с меткой {0,0}
-- Ожидается ошибка доступа
\connect - u_0_00
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_0_01 с меткой {0,1}
-- Ожидается ошибка доступа
\connect - u_0_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_1_01 с меткой {1,1}
-- Ожидается успех
\connect - u_1_01
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_0_10 с меткой {0,1}
-- Ожидается ошибка доступа
\connect - u_0_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_1_10 с меткой {1,2}
-- Ожидается ошибка доступа
\connect - u_1_10
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_1_11 с меткой {1,3}
-- Ожидается ошибка доступа
\connect - u_1_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем u_2_11 с меткой {2,3}
-- Ожидается ошибка доступа
\connect - u_2_11
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
-- Попытка обновления пользователем dbsa с флагами игнорирования МПРД
-- Ожидается успех
\connect - dbsa
UPDATE "Проверка" SET "данные" = "данные" || '+' || USER;
--
--
-- Ожидается, что изменить данные смогли только пользователи postgres и dbsa с установленными флагами игнорирования МПРД
-- и пользователь с меткой {1,1}
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Проверяем возможность обновления метки пользователем с флагом CHMAC
\connect - dbsa
--
-- Проверяем исходные метки
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Попытка явно изменить метки
-- Ожидается успех
UPDATE "Проверка" SET maclabel = '{1,2}', "данные" = "данные" || '+CHMAC' WHERE maclabel > '{0,0}';
--
-- Проверяем, что метка у строк изменилась
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";
--
-- Администратор postgres сбрасывает метку у таблицы
\connect - postgres
ALTER TABLE "Проверка" SET MAC TO NULL;
--
-- Проверяем возможность обновления метки пользователем без флага CHMAC
-- Ожидается ошибка доступа
\connect - u_0_00
UPDATE "Проверка" SET maclabel = '{0,0}', "данные" = "данные" || '+CHMAC';
--
-- Проверяем, что строки не были обновлены
SELECT maclabel,* FROM "Проверка" ORDER BY "идентификатор";

--
\i 'support/done.sql'
