-- RBT-TEST - Конфигурационные параметры
-- Проверка начальных установок и конфигурационных параметров
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
-- 
-- Проверяем, что сразу после создания базы данных защищена только таблица pg_largeobject
-- Ожидается одна запись pg_largeobject
SELECT relname AS "Наименование таблицы", relhasmacs AS "Записи имеют метки"
FROM pg_class WHERE relhasmacs = true;
--
-- Проверка конфигурационных параметров
-- Подключается пользователь dbsa
\connect - dbsa
--
-- Просмотр всех конфигурационных параметров
-- Ожидается 17 параметров
-- Привелегии пользователя (ac_capable_ignmaccat, ac_capable_chmac, ac_capable_ignmaclvl, ac_capable_mac_readsearch, ac_capable_setmac)
-- Настройки сервера (ac_caps_ttl, ac_debug_print, ac_enable_copy_to_file, ac_enable_truncate, ac_enable_trusted_owner,
--                    ac_ignore_server_maclabel, ac_ignore_socket_maclabel, ac_enable_sequence_mac, ac_enable_dblink_mac)
-- Уровни сессии и пользователя(ac_session_maclabel, ac_user_max_maclabel, ac_user_min_maclabel )
-- Версия модели МРД (ac_model_version)
--
SELECT name AS "Наименование параметра" FROM pg_settings WHERE name like 'ac_%' ORDER BY name;
--
-- Изменения значений параметра ac_session_maclabel
-- Изначально ac_session_maclabel = ac_user_max_maclabel
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel' or name = 'ac_user_max_maclabel';
--
-- Пользователь dbsa с флагом CAP_CHMAC может менять метку сессии в допустимых пределах
SELECT set_config ('ac_session_maclabel', (SELECT setting FROM pg_settings WHERE name = 'ac_user_min_maclabel'), false);
--
-- Проверка значений параметров ac_session_maclabel
-- Теперь ac_session_maclabel = ac_user_min_maclabel
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel' or name = 'ac_user_min_maclabel';
--
-- Попытка изменить параметр прямым запросом
UPDATE pg_settings SET setting = (SELECT setting FROM pg_settings WHERE name = 'ac_user_max_maclabel') WHERE name = 'ac_session_maclabel';
--
-- Проверка значений параметров ac_session_maclabel
-- Теперь ac_session_maclabel = ac_user_max_maclabel
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel' or name = 'ac_user_max_maclabel';
--
-- Пользователь dbsa с флагом CAP_CHMAC не может менять метку сессии вне допустимых пределов
-- Ожидается ошибка доступа
SELECT set_config ('ac_session_maclabel', '{8,0}', false);
--
-- Проверка значений параметров ac_session_maclabel
-- Ожидается, что значение ac_session_maclabel не изменилось
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel';
--
-- Проверка значений параметров ac_capable_chmac
-- Ожидается yes
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_capable_chmac';
--
-- Подключается пользователь u_0_00 без с флага CAP_CHMAC
\connect - u_0_00
--
-- Проверка значений параметров ac_capable_chmac
-- Ожидается no
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_capable_chmac';
--
-- Проверка значений параметров ac_session_maclabel
-- Изначально ac_session_maclabel = ac_user_max_maclabel
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel' or name = 'ac_user_max_maclabel';
--
-- Пользователь u_0_00 без флага CAP_CHMAC не может менять метку сессии
-- Ожидается ошибка доступа
SELECT set_config ('ac_session_maclabel', '{1,2}', false);
--
-- Проверка значений параметров ac_session_maclabel
-- Ожидается, что значение ac_session_maclabel не изменилось
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel';
--
-- Попытка изменить параметр прямым запросом
-- Ожидается ошибка доступа
UPDATE pg_settings SET setting = '{1,2}' WHERE name = 'ac_session_maclabel';
--
-- Проверка значений параметров ac_session_maclabel
-- Ожидается, что значение ac_session_maclabel не изменилось
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_session_maclabel';
--
-- Проверка процедуры конфигурирования сервера
--
-- Подключается администратор кластера postgres
\connect - postgres
--
-- Администратор кластера postgres устанавливает конфигурационный параметр ac_enable_sequence_mac
SELECT ac_change_param('ac_enable_sequence_mac', true);
--
-- небольшая задержка
\! sleep 1
--
-- Проверка значений параметра ac_enable_sequence_mac
-- Ожидается, что параметр ac_enable_sequence_mac установлен
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_enable_sequence_mac';
--
-- Администратор кластера postgres сбрасывает конфигурационный параметр ac_enable_sequence_mac
SELECT ac_change_param('ac_enable_sequence_mac', false);
--
-- небольшая задержка
\! sleep 1
--
-- Проверка значений параметра ac_enable_sequence_mac
-- Ожидается, что параметр ac_enable_sequence_mac сброшен
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_enable_sequence_mac';
--
-- Проверка значений параметра ac_model_version
-- Ожидается SE
SELECT name AS "Наименование параметра", setting AS "Значение" FROM pg_settings WHERE name = 'ac_model_version';
--
\i 'support/done.sql'
