-- RBT-TEST - Проверка pg_dump/pg_restore
-- Проверка резервирования/восстановления с помощью pg_dump,psql,pg_restore
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB" (loid oid, fd integer);
GRANT ALL ON "Проверка BLOB" TO PUBLIC;
--
-- Подключаемся как администратор кластера
\connect - postgres
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Вставляем запись в таблицу pg_largeobject с меткой {0,0} для объекта 200000
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200000, 0, 'test');
SELECT maclabel, * FROM pg_largeobject;
--
-- Подключаемся как пользователь с меткой {1,2}
\connect - u_1_10
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Создаем объект с идентификатором 200000 из таблицы pg_largeobject
-- ожидается ошибка доступа
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200000);
--
-- Создаем объект с идентификаторм 200001
-- Ожидается успех
INSERT INTO "Проверка BLOB" (loid) SELECT lo_create(200001);
SELECT * FROM "Проверка BLOB";
--
-- Делаем доступным всем
GRANT ALL ON LARGE OBJECT 200001 TO PUBLIC;
--
-- Начинаем транзакцию
BEGIN;
--
-- Открываем объект на чтение и запись
--   INV_READ  = 0x40000
--   INV_WRITE = 0x20000
UPDATE "Проверка BLOB" SET fd = lo_open(loid, CAST(x'20000' | x'40000' AS integer));
--
-- Проверка записи, ожидается успех
SELECT lowrite(fd, '
Whose woods these are I think I know,
His house is in the village though.
He will not see me stopping here,
To watch his woods fill up with snow.

My little horse must think it queer,
To stop without a farmhouse near,
Between the woods and frozen lake,
The darkest evening of the year.

He gives his harness bells a shake,
To ask if there is some mistake.
The only other sound''s the sweep,
Of easy wind and downy flake.

The woods are lovely, dark and deep,
But I have promises to keep,
And miles to go before I sleep,
And miles to go before I sleep.

         -- Robert Frost
') FROM "Проверка BLOB";
--
-- Закрываем объект
-- Ожидается 0 - отсутствие ошибок, отрицательное значение - ошибка
SELECT lo_close(fd) FROM "Проверка BLOB";
--
-- Завершаем транзакцию
END;
--
-- Проверяем прямой доступ к данным объектов
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект приобрел метку пользователя {1,2}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
--
-- Проверяем возможность работы с объектом пользователями с другими метками
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB0" (loid oid, fd integer);
INSERT INTO "Проверка BLOB0" values(200001, 0);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что пользователь с меньшей меткой, не может увидеть объект 200001
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся открыть объект на чтение
-- Ожидается ошибка, объект для пользователя недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'40000' AS integer));
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка, объект для пользователя опять же недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка, объект для пользователя недоступен
SELECT lo_unlink(loid) from "Проверка BLOB0";
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект виден для пользователя с большей меткой
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся добавить страницу к объекту
-- ожидается ошибка доступа
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, convert_to(USER || ' data', 'UTF8'));
--
-- Создаем тестовую таблицу
CREATE TABLE "Проверка BLOB2" (loid oid, fd integer);
INSERT INTO "Проверка BLOB2" values(200001, 0);
--
-- Пытаемся открыть объект на чтение
-- Ожидается успех
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB2";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB2";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB2";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB2";
SELECT lo_tell(fd) FROM "Проверка BLOB2";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB2";
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка доступа
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB2";
--
-- Выполняем резервирование с помощью утилиты pg_dump в текстовом режиме
\! /usr/lib/postgresql/9.2/bin/pg_dump -U postgres mtest > /tmp/dump
--
-- Просматриваем результат резервирования
\! cat /tmp/dump
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты psql
\! /usr/lib/postgresql/9.2/bin/psql -U postgres -d mtest -f /tmp/dump
--
\connect mtest
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
--
-- Проверяем прямой доступ к данным объектов
-- Ожидается, что объект имеет метку {1,2}
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект имеет метку {1,2}
SELECT maclabel, oid FROM pg_largeobject_metadata;
--
--
-- Проверяем возможность работы с объектом пользователями с другими метками
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
INSERT INTO "Проверка BLOB0" values(200001, 0);
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что пользователь с меньшей меткой, не может увидеть объект 200001
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся открыть объект на чтение
-- Ожидается ошибка, объект для пользователя недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'40000' AS integer));
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка, объект для пользователя опять же недоступен
BEGIN;
UPDATE "Проверка BLOB0" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка, объект для пользователя недоступен
SELECT lo_unlink(loid) from "Проверка BLOB0";
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- Устанавливаем режим вывода типа bytea
SET bytea_output TO escape;
--
-- Проверяем атрибуты объекта
-- Ожидается, что объект виден для пользователя с большей меткой
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Пытаемся добавить страницу к объекту
-- ожидается ошибка доступа
INSERT INTO pg_largeobject(loid, pageno, data) VALUES(200001, 1, convert_to(USER || ' data', 'UTF8'));
--
INSERT INTO "Проверка BLOB2" values(200001, 0);
--
-- Пытаемся открыть объект на чтение
-- Ожидается успех
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'40000' AS integer));
--
-- Устанавливаем позицию начала чтения
SELECT lo_lseek(fd, 422, 0) FROM "Проверка BLOB2";
--
-- Читаем 35 символов
SELECT loread(fd, 35) FROM "Проверка BLOB2";
--
-- Меняем позицию начала чтения
SELECT lo_lseek(fd, -19, 1) FROM "Проверка BLOB2";
--
-- Текущий размер
SELECT lo_lseek(fd, 0, 2) FROM "Проверка BLOB2";
SELECT lo_tell(fd) FROM "Проверка BLOB2";
--
-- Закрываем объект и завершаем транзакцию
SELECT lo_close(fd) FROM "Проверка BLOB2";
END;
--
-- Пытаемся открыть объект на запись
-- Ожидается ошибка доступа
BEGIN;
UPDATE "Проверка BLOB2" SET fd=lo_open(loid, CAST(x'20000' AS integer));
END;
--
-- Пытаемся удалить объект
-- Ожидается ошибка доступа
SELECT lo_unlink(loid) from "Проверка BLOB2";
--
-- Удаляем временный файл
\! /bin/rm -f /tmp/dump
--
-- Выполняем резервирование с помощью утилиты pg_dump в бинарном режиме
\! /usr/lib/postgresql/9.2/bin/pg_dump -U postgres mtest -F c -f /tmp/dump
--
-- Просматриваем результат резервирования
\! lsm -Mn /tmp/dump
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
-- Выполняем восстановление с помощью утилиты psql
\! /usr/lib/postgresql/9.2/bin/pg_restore -h localhost -U postgres -d mtest /tmp/dump
--
\connect mtest
--
-- Отключаем ДПРД к таблице pg_largeobject
GRANT ALL ON pg_largeobject TO PUBLIC;
--
-- Проверяем результат восстановления
\i 'support/select-test-table.sql'
--
-- Проверяем атрибуты объекта
SELECT maclabel, oid FROM pg_largeobject_metadata;
SELECT maclabel, loid, pageno, substring(data from 1 for 10) FROM pg_largeobject;
--
-- Удаляем временный файл
\! /bin/rm -f /tmp/dump
--
-- Пересоздаем базу данных
\connect template1 postgres
DROP DATABASE mtest;
CREATE DATABASE mtest;
--
\connect mtest
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
--
-- Устанавливаем признак использования меток столбцов
\connect - postgres
ALTER TABLE "Проверка" ENABLE COLUMN MACS;
ALTER TABLE "Проверка" ALTER COLUMN "данные" SET MAC TO '{1,1}';
--
-- Выполняем резервирование с помощью утилиты pg_dump в текстовом режиме
\! /usr/lib/postgresql/9.2/bin/pg_dump --table="Проверка" -h localhost -U postgres mtest > /tmp/dump
--
-- Просматриваем результат резервирования
\! cat /tmp/dump
--
-- Удаляем таблицу
\connect - postgres
DROP TABLE "Проверка";
--
-- Выполняем восстановление с помощью утилиты psql
\! /usr/lib/postgresql/9.2/bin/psql -h localhost -U postgres -d mtest -f /tmp/dump
--
-- Удаляем временный файл
\! /bin/rm -f /tmp/dump
--
-- Проверяем результат восстановления
\d+ "Проверка"
--
SELECT attname, attmaclabel FROM pg_catalog.pg_attribute
WHERE attrelid = (select oid from pg_catalog.pg_class where relname = 'Проверка')
and attnum >= 0;
--
\i 'support/done.sql'
