-- RBT-TEST - МПРД при использовании динамических запросов
-- Проверка сохранения работы МПРД при использовании динамических запросов
--
\i 'support/init.sql'
\i 'support/create-test-table.sql'
\i 'support/init-test-table.sql'
\i 'support/init-plperl.sql'
\i 'support/select-test-table.sql'
--
\connect - postgres
--
-- Администратор postgres создает процедуру transform_query
-- В результате выполнения функции должен быть создан план выполнения трансформированного запроса
-- (вместо создания плана можно возвращать сформированный текст запроса, но создание плана теоретически
--  суть быстрее за счет сокращения взаимодействия и за счет итоговой проверки запроса в процессе его
--  подготовки, а не последующего запуска)
CREATE FUNCTION transform_query (TEXT, TEXT) RETURNS TEXT AS $$
DECLARE
	query_plan        ALIAS FOR $1;
	original_query    ALIAS FOR $2;
	transformed_query TEXT;
BEGIN
	transformed_query = 'SELECT count(*) AS "' || query_plan || '" from (' || original_query || ') a';

	EXECUTE 'PREPARE ' || query_plan || ' AS ' || transformed_query;
	RETURN transformed_query;
END;
$$ LANGUAGE plpgsql SECURITY INVOKER;
--
-- Выполняем запрос от имени суперпользователя postgres
-- Ожидается 10
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_0_00 с меткой {0,0}
-- Ожидается 2
\connect - u_0_00
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_0_01 с меткой {0,1}
-- Ожидается 3
\connect - u_0_01
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_1_01 с меткой {1,1}
-- Ожидается 5
\connect - u_1_01
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_0_10 с меткой {0,2}
-- Ожидается 3
\connect - u_0_10
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_1_10 с меткой {1,2}
-- Ожидается 5
\connect - u_1_10
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_1_11 с меткой {1,3}
-- Ожидается 9
\connect - u_1_11
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени пользователя u_2_11 с меткой {2,3}
-- Ожидается 10
\connect - u_2_11
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
-- Выполняем запрос от имени администратора dbsa
-- Ожидается 10
\connect - dbsa
SET ac_session_maclabel = '{0, 0}';
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
SET ac_session_maclabel = '{1, 0}';
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
SET ac_session_maclabel = '{1, 3}';
SELECT transform_query ('query', 'SELECT "идентификатор" FROM "Проверка"'); EXECUTE query; DEALLOCATE query;
--
\i 'support/done.sql'
